/**************************************************************************
 *	Copyright (C) 2004-2007  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "fe_include.hpp"
#include "fe_config.hpp"

XERCES_CPP_NAMESPACE_USE;

void FeConfig::startElement(const   XMLCh* const    uri, 
							const   XMLCh* const    localname,
							const   XMLCh* const    qname, 
							const   Attributes&		attributes)
{
    char* message = XMLString::transcode(localname);

	if(strcmp(message, "matches") == 0)
	{
		//TODO This should actually check if the file is an absolute path or not.
	    char* file = XMLString::transcode(attributes.getValue(XMLString::transcode("file")));
		m_MatchFile = m_DataDir + file;
		XMLString::release(&file);
	}
	else if(strcmp(message, "root-node") == 0)
	{
		char* tex_enable = XMLString::transcode(attributes.getValue(XMLString::transcode("tex_enabled")));
	    char* tilt = XMLString::transcode(attributes.getValue(XMLString::transcode("axial_tilt")));
		char* tex = XMLString::transcode(attributes.getValue(XMLString::transcode("texture")));
		char* axis = XMLString::transcode(attributes.getValue(XMLString::transcode("axis")));

		m_RootTex = m_DataDir + tex;
		m_RootTilt = (float)atof(tilt);

		XMLString::release(&axis);
		XMLString::release(&tex);
		XMLString::release(&tilt);
		XMLString::release(&tex_enable);

	}

	XMLString::release(&message);
}

void FeConfig::endElement( const XMLCh* const uri,
					 const XMLCh* const localname,
					 const XMLCh* const qname)
{
//    char* message = XMLString::transcode(name);
}

void FeConfig::endDocument()
{

}

void FeConfig::fatalError(const SAXParseException& exception)
{
    char* message = XMLString::transcode(exception.getMessage());
    cerr << "Fatal Error: " << message
         << " at line: " << exception.getLineNumber()
         << endl;
         
    XMLString::release(&message);
}

bool FeConfig::LoadConfig()
{
	m_wxConfig->Read(_("/UI/Window/X"), &m_Width, 640);
	m_wxConfig->Read(_("/UI/Window/Y"), &m_Height, 480);
		
	return true;
}

bool FeConfig::SaveConfig()
{
	m_wxConfig->Write(_("/UI/Window/X"), m_Width);
	m_wxConfig->Write(_("/UI/Window/Y"), m_Height);	
	return true;
}

/*********************
  Construction
**********************/
FeConfig::FeConfig()
{ 
	// TODO this has to be changed
	m_DataDir		= FE_DATA_DIR;
	enableEarth		= true;
	
	m_wxConfig		= new wxConfig(_("fe3d"));
		
	this->LoadConfig();
}

/*********************
   Destruction
**********************/
FeConfig::~FeConfig()
{
	this->SaveConfig();
	delete m_wxConfig;
}
