/**************************************************************************
 *	Copyright (C) 2004-2007  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "fe_include.hpp"

#include "fe_core.hpp"
#include "fe_tools.hpp"
#include "fe_events.hpp"
#include "fe_xml_util.hpp"
#include "gl_util.hpp"
#include "nmap_parser.hpp"

//#ifdef HAVE_DIRENT
//# include <dirent.h>
//#endif 

using namespace std;
using namespace ICMATH;
XERCES_CPP_NAMESPACE_USE

// Where should these go?
GLfloat light_position[]	= { 1.0f, 1.0f, 1.0f, 0.0 };
GLfloat light_position2[]	= { -1.0f, -1.0f, -1.0f, 0.0 };
GLfloat white_light[] 		= { 4.0f, 4.0f, 4.0f, 1.0f };

/***************************************
 * Initialization
 ***************************************/
bool FeCore::init()
{
	try
	{
		XMLPlatformUtils::Initialize();
	}
	catch (const XMLException& toCatch)
	{
		char* message = XMLString::transcode(toCatch.getMessage());
		cerr << "Error during initialization! :"<<endl<<message<<endl;
		XMLString::release(&message);
		return false;
	}

	if(!this->parseFile(m_Config, this->getConfigFile("fe_config.xml")))
	{
		cerr<<"Failed to load configuration file"<<endl;
		return false;
	}

	m_Matcher = FeMatchParser::instance();
	m_Matcher->setDataDir(m_Config->getDataDir().c_str());
	m_Matcher->setTextureCache(m_TexCache);

	if(!this->parseFile(m_Matcher, m_Config->getMatchFile()))
	{
		cerr<<"Failed to load matches file"<<endl;
		return false;
	}
	Geo *g = 0;

	m_GeoCache->addGeometry("rootSphere", FeDraw::getGeo_Sphere(0.1f));
	m_GeoCache->addGeometry("tinySphere", FeDraw::getGeo_Sphere(0.009f, 8));
	m_GeoCache->addGeometry("fatSphere", FeDraw::getGeo_Sphere(0.03f, 8));
	m_GeoCache->addGeometry("ipLabel", FeDraw::getGeo_IP());

	g = FeDraw::getGeo_Square(0.1f, 0.1f);
	g->setRotation(-20.0f, 1.0f, 0.0f, 0.0f);
	g->setTransform(0.0f, -0.01f, 0.06f);

	m_GeoCache->addGeometry("generic", g);

	g = FeDraw::getGeo_Cylinder(90, 0.06f, 0.06f, false);
	g->setRotation(90.0f, 0.0f, 1.0f, 0.0f);
	m_GeoCache->addGeometry("router", g);

	g = FeDraw::getGeo_Cylinder(90, 0.1f, 0.06f, true);
	g->setRotation(90.0f, 0.0f, 1.0f, 0.0f);
	m_GeoCache->addGeometry("scanner", g);

	m_GeoCache->addGeometry("unknown", FeDraw::getGeo_Sphere(0.08f));
//	m_GeoCache->addGeometry("rotationWidget", FeDraw::getGeo_WireCircle(45, 0.1f));

	m_GeoCache->addGeometry("treeCtrl-CylOut", FeDraw::getGeo_Cylinder(20, 0.03f, 0.01f, true));
	m_GeoCache->addGeometry("treeCtrl-CylIn", FeDraw::getGeo_Cylinder(20, 0.022f, 0.01f, true));
	m_GeoCache->addGeometry("treeCtrl-CylCap", FeDraw::getGeo_Circle(20, 0.03f, 0.022f));
	m_GeoCache->addGeometry("treeCtrl-MinusCap", FeDraw::getGeo_Square(0.03f, 0.009f));
	m_GeoCache->addGeometry("treeCtrl-Minus", FeDraw::getGeo_Box(0.03f, 0.009f, 0.01f));

//	m_GeoCache->addGeometry("treeCtrl-PlusCap", FeDraw::getGeo_Circle(8, 0.4f, 0.1f));
//	m_GeoCache->addGeometry("treeCtrl-Plus", FeDraw::getGeo_Circle(8, 0.4f, 0.1f));

	m_TexCache->addImageTexture(new TexImage("rootTex", m_Config->getRootTex().c_str()));
	return true;
}

bool FeCore::initGL()
{
	m_Width = m_Config->getWidth();
	m_Height = m_Config->getHeight();

	glViewport(0, 0, m_Width, m_Height);
	glShadeModel(GL_SMOOTH);
	glClearColor(0.0f, 0.0f, 0.0f, 0.5f);

	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, white_light);
	glLightfv(GL_LIGHT0, GL_SPECULAR, white_light);

	glLightfv(GL_LIGHT1, GL_POSITION, light_position2);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, white_light);
	glLightfv(GL_LIGHT1, GL_SPECULAR, white_light);

	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHT1);
	
	glClearDepth(1.0f);										// Depth Buffer Setup
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LESS);
    glMatrixMode(GL_PROJECTION);

	glLoadIdentity();

	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glClearColor(0.0, 0.0, 0.0, 1.0);

	glEnable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);

    gluPerspective(45.0f,(GLfloat)m_Width/(GLfloat)m_Height, 0.1f, 1000.0f);	// Calculate The Aspect Ratio Of The Window
    glMatrixMode(GL_MODELVIEW);

	glLoadIdentity();

	return true;
}

/***************************************
 * Scan Processing
 ***************************************/
bool FeCore::loadScan(const char* file, FeHostsNode *node)
{
	cout<<"Loading scan file: "<<file<<endl;
	
	NmapParser* nmapParser = new NmapParser(node);
	bool ret = parseFile(nmapParser, file);

	if(node->getSubNodeCount() > 0)
	{
		m_RootNode->addNode(node);
		updateRadius();
	}
	else
	{
		cerr<<"Failed to load scan file: "<<file<<endl;
		delete node;	
	}

	delete nmapParser;
	return ret;
}


// TODO Change this to use wxFileName and not hand-crafted output files
FeTool* FeCore::spawnNmap(vector<wxString>* command, wxString label)
{
	FeTool *tool = new FeTool(_("nmap"), _("nmap"), _(""));

	// Construct the output file
	wxString outfile = wxString::Format(_T("nm-scan-%i.xml"), (int) wxDateTime::Now().GetTicks());
	wxFileName outFile(wxFileName::GetTempDir(), outfile);

	tool->setOutputArg(wxT("-oX"));
	tool->setOutputFile(outFile.GetFullPath());
	
	if(!tool->launch(command))
		return 0;

#ifdef UNICODE
	FeHostsNode* sNode = new FeHostsNode(label.ToAscii().data(), tool);
#else
	FeHostsNode* sNode = new FeHostsNode(label.c_str(), tool);
#endif

	m_RootNode->addNode(sNode);

	return tool;
}

/***************************************
 * Rendering
 ***************************************/
void FeCore::render(bool select)
{
	glTranslatef(0.0f,y_Trans,z_Trans);
	glRotatef(rtri,0.0f,-0.5f,0.0f);
	glRotatef(x_Rot, 1, 0, 0);
	glRotatef(y_Rot, 0, 1, 0);

	if(!select)
		m_RootNode->render();
	else
		m_RootNode->RenderSelect();
}

void FeCore::enablePicking(unsigned int x, unsigned int y)
{
	GLint viewport[4];

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); // ? 
	glSelectBuffer(SELECT_BUFSIZE,selectBuf);
	glGetIntegerv(GL_VIEWPORT,viewport);
	glRenderMode(GL_SELECT);
	glInitNames();

	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();

	gluPickMatrix(x,viewport[3]-y, 3,3,viewport);
	gluPerspective(45.0f,(GLfloat)viewport[2]/(GLfloat)viewport[3],0.1f,100.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

}

bool FeCore::disablePicking()
{
	int hits = 0;
	topHitName = 0;

	glMatrixMode(GL_PROJECTION);
	glPopMatrix();

	hits = glRenderMode(GL_RENDER);
	glMatrixMode(GL_MODELVIEW);

	// TODO Clean this up
	if(hits != 0)
	{
		unsigned int names[15];
		MouseEvent *mEvent = 0;
		unsigned int cn = processHits(hits, selectBuf, names);

		int thisClick = lastClick = 0;
		if((thisClick-lastClick) > 600)
		{
			lastClick=thisClick;

			if(cn == prevHitCount){
				bool hitMatch = true;

				for(unsigned int i = 0; i < cn; ++i)
				{
					if(prevHits[i] != names[i]){
						hitMatch = false;
						break;
					}
				}

				if(hitMatch){
					m_NodeSelect.m_HitCount = cn;
					m_NodeSelect.m_Hits = names;

					// TEST
					topHitName = names[0];

					MouseEvent *mEvent = new MouseEvent(0,0, 0, ME_dclick, ME_none);
					mEvent->setHitData(cn, names);
					pushEvent(mEvent);
					delete mEvent;
					return true;
				}
			}
		}

		prevHitCount = cn;

		for(unsigned int i = 0; i < cn; ++i)
			prevHits[i] = names[i];

		ME_Mod mod = ME_none;

		if(ctrlDown)
			mod = ME_ctrl;

		mEvent = new MouseEvent(0,0, 0, ME_click, mod);
		mEvent->setHitData(cn, names);
		pushEvent(mEvent);
		delete mEvent;
		return true;
	}

	return false;
}

int FeCore::processHits (GLuint hits, GLuint buffer[], GLuint *ns)
{
	unsigned int i, j;
	GLuint names, *ptr, minZ,*ptrNames, numberOfNames;

//	printf ("hits = %d\t", hits);
	ptr = (GLuint *) buffer;
	minZ = 0xffffffff;
	for (i = 0; i < hits; i++) {	
		names = *ptr;
		ptr++;
		if (*ptr < minZ) {
			numberOfNames = names;
			minZ = *ptr;
			ptrNames = ptr+2;
		}
		
		ptr += names+2;
	}

//	printf ("Hit names: ");
	ptr = ptrNames;
	for (j = 0; j < numberOfNames; ++j,++ptr) {
//		printf ("%d ,", *ptr);
		ns[j] = *ptr;
	}
//	printf ("\n");

  return numberOfNames;
}

bool FeCore::LoadFile(wxString file)
{

	return false;
}

bool FeCore::SaveFile(wxString file)
{
	wxXmlDocument doc;
	doc.SetRoot(this->m_RootNode->GetXMLNode());
	doc.Save(file);

	return true;
}


/***************************************
 * Settings
 ***************************************/
bool FeCore::parseFile(DefaultHandler* handler, string file)
{
	if(file.size() <= 0)
	{
		cerr<<"FeCore::parseFile : Can not load empty file"<<endl;
		return false;
	}

    SAX2XMLReader* parser	= XMLReaderFactory::createXMLReader();
	int errorCount			= 0;
    int errorCode			= 0;

    try
    {
        parser->setContentHandler(handler);
        parser->setErrorHandler(handler);
        parser->parse(file.c_str());
        errorCount = parser->getErrorCount();
    }
    catch (const OutOfMemoryException&)
    {
        cerr<< "OutOfMemoryException"<<endl;
		::wxMessageBox(_("Out of Memory Exception"), _("Out of Memory"), 
					wxOK | wxICON_ERROR, 0);
        errorCode = 5;          
    }
    catch (const XMLException& toCatch)
    {
		char* err = XMLString::transcode(toCatch.getMessage());
        cerr << "\nAn error occurred\n  Error: "<< err<<endl;
		::wxMessageBox(wxString::FromAscii(err), _("Parse Error"), 
					wxOK | wxICON_ERROR, 0);
        errorCode = 4;
		XMLString::release(&err);
    }
	catch (const SAXNotRecognizedException* toCatch)
	{
		char* err = XMLString::transcode(toCatch->getMessage());
        cerr << "\nAn error occurred\n  Error: "<<err<<endl;

		::wxMessageBox(wxString::FromAscii(err), _("Parse Error"), 
						wxOK | wxICON_ERROR, 0);
        errorCode = 4;
		XMLString::release(&err);
	}
	catch (...)
	{
		cerr<<"Parsing Exception"<<endl;
	}

	delete parser;
	
	// TODO this should be graceful
    if(errorCode == 5) {
		XMLPlatformUtils::Terminate();
        return false;
    }

	return true;
}

string FeCore::getConfigFile(string file) 
{
	return FE_DATA_DIR + file;
}

/***************************************
 * Misc
 ***************************************/
int FeCore::screenShot()
{
	return 0;
}

void FeCore::updateRadius()
{
	m_RootNode->updateRadius();
}

bool FeCore::pushEvent(Event *e)
{
	return m_RootNode->pushEvent(e);
}

void FeCore::clearScene()
{
	m_RootNode->clear();
}

void FeCore::setWindowSize(int w, int h)
{
	m_Width = w;
	m_Height = h;
	
	m_Config->setWindowSize(w,h);

}

/***************************************
 * Construction / Destruction
 ***************************************/
FeCore::FeCore()
{
	m_Width		= 0;
	m_Height	= 0;
	rtri		= 0;
	x_Rot		= 0;
	y_Rot		= 0;
 	y_Trans		= 0.6f;
	z_Trans		= -2.0f;

	m_Config	= new FeConfig();
	m_GeoCache	= GeometryCache::instance();
	m_TexCache	= TextureCache::instance();
	
	m_RootNode	= new FeRootNode(m_Config);
	m_Matcher	= 0;
	
	// Kill these off
	mouseDownLeft	= mouseDownRight = false;
	underSelection	= false;
	ctrlDown		= false;
	shiftDown		= false;

	prevX = 0;
	prevY = 0;
}

FeCore::~FeCore()
{
	delete m_RootNode;
	delete m_Matcher;
	delete m_TexCache;
	delete m_GeoCache;
	delete m_Config;
	XMLPlatformUtils::Terminate();	
}
