/**************************************************************************
 *	Copyright (C) 2004-2007  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "fe_include.hpp"
#include "fe_draw.hpp"

using namespace std;
using namespace ICMATH;

Geo* FeDraw::getGeo_Square(float width, float height)
{
	Geo *g = new Geo(GL_T2F_N3F_V3F, GL_QUADS, 4);
	int s = 0;

	g->data[s++] = 0.0f;	// Texture  // 1.0f
	g->data[s++] = 0.0f;				// 0.0f

	g->data[s++] = 0.0f;	// Normals
	g->data[s++] = 0.0f;
	g->data[s++] = 1.0f;
	g->data[s++] = -width/2.0f;	// Vertex
	g->data[s++] = height/2.0f;
	g->data[s++] = 0.0f;

	g->data[s++] = 1.0f;	// Texture // 1.0f
	g->data[s++] = 0.0f;				// 1.0f

	g->data[s++] = 0.0f;	// Normals
	g->data[s++] = 0.0f;
	g->data[s++] = 1.0f;
	g->data[s++] = width/2.0f;	// Vertex
	g->data[s++] = height/2.0f;
	g->data[s++] = 0.0f;

	g->data[s++] = 1.0f;	// Texture		// 0.0f
	g->data[s++] = 1.0f;					// 1.0f

	g->data[s++] = 0.0f;	// Normals
	g->data[s++] = 0.0f;
	g->data[s++] = 1.0f;
	g->data[s++] = width/2.0f;	// Vertex
	g->data[s++] = -height/2.0f;
	g->data[s++] = 0.0f;

	g->data[s++] = 0.0f;	// Texture		// 0.0f
	g->data[s++] = 1.0f;					// 0.0f

	g->data[s++] = 0.0f;	// Normals
	g->data[s++] = 0.0f;
	g->data[s++] = 1.0f;
	g->data[s++] = -width/2.0f;	// Vertex
	g->data[s++] = -height/2.0f;
	g->data[s++] = 0.0f;

	return g;
}
Geo* FeDraw::getGeo_WireCircle(int n, float radius)
{
	Geo *g = new Geo(GL_V3F, GL_LINE_LOOP, n + 1);

	float x = 0;
	float y = 0;
	float step = (float)360/n;
	
	int s = 0;
	
	for(float j = 0; j <= n; ++j)
	{
		x=radius*cos((float)((PI/180)*j*step));
		y=radius*sin((float)((PI/180)*j*step));

		// Verteces
        *(g->data + s + 0) = x;
        *(g->data + s + 1) = 0.0f;
        *(g->data + s + 2) = y;
		s += 3;
	}
	
	g->setRotation(90.0f, 0.0f, 1.0f, 0.0f);
	return g;
}

Geo* FeDraw::getGeo_Circle(int n, float outRadius, float inRadius)
{
	Geo *g = new Geo(GL_N3F_V3F, GL_TRIANGLE_STRIP, 2*n + 2);

	float x_out = 0;
	float y_out = 0;

	float x_in	= 0;
	float y_in	= 0;

	float step = (float)360/n;
	
	int s = 0;
	
	for(float j = 0; j <= n; ++j)
	{
		x_out=outRadius*cos((float)((PI/180)*j*step));
		y_out=outRadius*sin((float)((PI/180)*j*step));

		x_in=inRadius*cos((float)((PI/180)*j*step));
		y_in=inRadius*sin((float)((PI/180)*j*step));


        *(g->data + s + 0) = 0.0f;	// Normal
        *(g->data + s + 1) = 1.0f;
        *(g->data + s + 2) = 0.0f;

        *(g->data + s + 3) = x_out;	// Vertex
        *(g->data + s + 4) = 0.0f;
        *(g->data + s + 5) = y_out;

        *(g->data + s + 6) = 0.0f;	// Normal
        *(g->data + s + 7) = 1.0f;
        *(g->data + s + 8) = 0.0f;

        *(g->data + s + 9) = x_in;	// Vertex
        *(g->data + s + 10) = 0.0f;
        *(g->data + s + 11) = y_in;

		s += 12;
	}

	return g;
}
Geo* FeDraw::getGeo_Box(float width, float height, float depth)
{
	Geo *g = new Geo(GL_N3F_V3F, GL_QUAD_STRIP, 10);

	int s = 0;

	g->data[s++] = -1.0f/SQRT_3;	// Normal
	g->data[s++] = -1.0f/SQRT_3;
	g->data[s++] = 1.0f/SQRT_3;

	g->data[s++] = -width/2.0f;		// Vertex
	g->data[s++] = -height/2.0f;
	g->data[s++] = depth/2.0f;

	g->data[s++] = -1.0f/SQRT_3;	// Normal
	g->data[s++] = -1.0f/SQRT_3;
	g->data[s++] = -1.0f/SQRT_3;

	g->data[s++] = -width/2.0f;		// Vertex
	g->data[s++] = -height/2.0f;
	g->data[s++] = -depth/2.0f;

	g->data[s++] = 1.0f/SQRT_3;		// Normal
	g->data[s++] = -1.0f/SQRT_3;
	g->data[s++] = -1.0f/SQRT_3;

	g->data[s++] = -width/2.0f;		// Vertex
	g->data[s++] = height/2.0f;
	g->data[s++] = depth/2.0f;

	g->data[s++] = -1.0f/SQRT_3;	// Normal
	g->data[s++] = 1.0f/SQRT_3;
	g->data[s++] = -1.0f/SQRT_3;

	g->data[s++] = -width/2.0f;		// Vertex
	g->data[s++] = height/2.0f;
	g->data[s++] = -depth/2.0f;

	g->data[s++] = 1.0f/SQRT_3;		// Normal
	g->data[s++] = 1.0f/SQRT_3;
	g->data[s++] = 1.0f/SQRT_3;

	g->data[s++] = width/2.0f;		// Vertex
	g->data[s++] = height/2.0f;
	g->data[s++] = depth/2.0f;

	g->data[s++] = 1.0f/SQRT_3;		// Normal
	g->data[s++] = 1.0f/SQRT_3;
	g->data[s++] = -1.0f/SQRT_3;

	g->data[s++] = width/2.0f;		// Vertex
	g->data[s++] = height/2.0f;
	g->data[s++] = -depth/2.0f;

	g->data[s++] = 1.0f/SQRT_3;		// Normal
	g->data[s++] = -1.0f/SQRT_3;
	g->data[s++] = 1.0f/SQRT_3;

	g->data[s++] = width/2.0f;		// Vertex
	g->data[s++] = -height/2.0f;
	g->data[s++] = depth/2.0f;

	g->data[s++] = 1.0f/SQRT_3;		// Normal
	g->data[s++] = -1.0f/SQRT_3;
	g->data[s++] = -1.0f/SQRT_3;

	g->data[s++] = width/2.0f;		// Vertex
	g->data[s++] = -height/2.0f;
	g->data[s++] = -depth/2.0f;

	g->data[s++] = -1.0f/SQRT_3;	// Normal
	g->data[s++] = -1.0f/SQRT_3;
	g->data[s++] = 1.0f/SQRT_3;

	g->data[s++] = -width/2.0f;		// Vertex
	g->data[s++] = -height/2.0f;
	g->data[s++] = depth/2.0f;

	g->data[s++] = 1.0f/SQRT_3;		// Normal
	g->data[s++] = 1.0f/SQRT_3;
	g->data[s++] = -1.0f/SQRT_3;

	g->data[s++] = -width/2.0f;	// Vertex
	g->data[s++] = -height/2.0f;
	g->data[s++] = -depth/2.0f;

	return g;
}

Geo* FeDraw::getGeo_Cylinder(int n, float radius, float height, bool full_texture)
{
	Geo *g = new Geo(GL_T2F_N3F_V3F, GL_QUAD_STRIP, 2*(n + 1));

	float x, y;
	float i = 0;
	
	int s = 0;	
	float step = (float)360/n;
	float mag = 0;

	GLfloat *vaRouter = g->data;
	

	for(float j = n; j >= 0; --j)
	{
		x=radius*cos((float)((PI/180)*j*step));
		y=radius*sin((float)((PI/180)*j*step));

		mag = sqrt(x*x + y*y);

		if(mag <= 0)
			mag = 1; 

// HACK: 
		// Textures;

		if(full_texture)
		{
				*(vaRouter + s) =  (float)(1.0 - (j*step/(360)));
				*(vaRouter + s + 1) = 1.0f;
//				++i;
		}
		else
		{
			if((j > 120/step) && (j <= 240/step))
			{
				*(vaRouter + s) =  (float)(i*step/(120));
				*(vaRouter + s + 1) = 1.0f;
				++i;
			}
			else	
			{
				*(vaRouter + s) = 0.0f;
			}
		}

        *(vaRouter + s + 8) = *(vaRouter + s);
		*(vaRouter + s + 9) = 0.0f;

		// Normals	
		*(vaRouter + s + 2) = x/mag;
        *(vaRouter + s + 3) = 0.0f;
        *(vaRouter + s + 4) = y/mag;

        *(vaRouter + s + 10) = *(vaRouter + s + 2);
        *(vaRouter + s + 11) = *(vaRouter + s + 3);
        *(vaRouter + s + 12) = *(vaRouter + s + 4);

		// Verteces
        *(vaRouter + s + 5) = x;
        *(vaRouter + s + 6) = 0.0f;
        *(vaRouter + s + 7) = y;

        *(vaRouter + s + 13) = x; 
        *(vaRouter + s + 14) = height;
        *(vaRouter + s + 15) = y;
	
		s += 16;
	}
	
	g->data = vaRouter;
	return g;
}


Geo* FeDraw::getGeo_Sphere(float r, int n)
{
	const int points = n*(n+1);

	Geo *g = new Geo(GL_T2F_N3F_V3F, GL_QUAD_STRIP, points);

	int offset = 0;
	int i,j;
	float theta1,theta2,theta3;
	xyz e,p,c;

	c.x = c.z = c.y = 0;

	if (r < 0) r = -r;
	if (n < 0) n = -n;
	if (n < 4 || r <= 0) {
//		glBegin(GL_POINTS);
//		glVertex3f(c.x,c.y,c.z);
//		glEnd();
		return NULL;
	}
	
	for (j=0; j<n/2; ++j) 
	{
		theta1 = j * TWOPI / n - PID2;
		theta2 = (j + 1) * TWOPI / n - PID2;

		for (i=0;i<=n;i++) 
		{
			theta3 = i * TWOPI / n;

			e.x = cos(theta2) * cos(theta3);
			e.y = sin(theta2);
			e.z = cos(theta2) * sin(theta3);
			p.x = c.x + r * e.x;
			p.y = c.y + r * e.y;
			p.z = c.z + r * e.z;

			g->data[offset++] = i/(float)n;			// Texture
			g->data[offset++] = 1.0 - 2*(j+1)/(float)n;

			g->data[offset++] = e.x;				// Normals
			g->data[offset++] = e.y;
			g->data[offset++] = e.z;

			g->data[offset++] = p.x;				// Vertex
			g->data[offset++] = p.y;
			g->data[offset++] = p.z;

			e.x = cos(theta1) * cos(theta3);
			e.y = sin(theta1);
			e.z = cos(theta1) * sin(theta3);
			p.x = c.x + r * e.x;
			p.y = c.y + r * e.y;
			p.z = c.z + r * e.z;

			g->data[offset++] = i/(float)n;			// Texture
			g->data[offset++] = 1.0 - 2*j/(float)n;

			g->data[offset++] = e.x;				// Normals
			g->data[offset++] = e.y;
			g->data[offset++] = e.z;

			g->data[offset++] = p.x;				// Vertex
			g->data[offset++] = p.y;
			g->data[offset++] = p.z;

		}
	}

	return g;
}


Geo* FeDraw::getGeo_IP()
{
		const float labelWidth = 0.03f;
		const float rad = 0.16f;
		const int	n = 15;
		const int slice = 90/n;

		Geo *g = new Geo(GL_T2F_N3F_V3F, GL_QUAD_STRIP, 2*n);

		float x		= 0.0f;
		float z		= 0.0f;
		float mag	= 0.0f;
		int s = 0;
	
		for(float j = 0; j < n ; ++j)
		{
			x=rad*cos((float)((PI/180)*j*slice));
			z=rad*sin((float)((PI/180)*j*slice));
			mag = sqrt(x*x + z*z);

			g->data[s++] = (float)(1 - j/n);
			g->data[s++] = 1.0f;
			g->data[s++] = (float)x/mag;
			g->data[s++] = 0.0f;
			g->data[s++] = (float)z/mag;
			g->data[s++] = (float)x;
			g->data[s++] = 0.0f;
			g->data[s++] = (float)z;

			g->data[s++] = (float)(1 - j/n);
			g->data[s++] = 0.0f;
			g->data[s++] = (float)x/mag;
			g->data[s++] = 0.0f;
			g->data[s++] = (float)z/mag;
			g->data[s++] = (float)x;
			g->data[s++] = labelWidth;
			g->data[s++] = (float)z;
		}
	return g;
}



// Various Rendering routines

void FeDraw::renderFoldWidget(bool plus)
{
	GeometryCache* geoCache = GeometryCache::instance();

	Geo* gCylIn		= geoCache->getGeometry("treeCtrl-CylIn");
	Geo* gCylOut	= geoCache->getGeometry("treeCtrl-CylOut");
	Geo* gCylCap	= geoCache->getGeometry("treeCtrl-CylCap");
	Geo* gMinus		= geoCache->getGeometry("treeCtrl-Minus");
	Geo* gMinusCap	= geoCache->getGeometry("treeCtrl-MinusCap");

	gCylOut->setRotation(90.0f, 1.0f, 0.0f, 0.0f);
	gCylIn->setRotation(90.0f, 1.0f, 0.0f, 0.0f);
	gCylCap->setRotation(90.0f, 1.0f, 0.0f, 0.0f);

	gCylIn->render();
	gCylOut->render();
	gCylCap->render();

// Draw Minus
	glPushMatrix();
	gMinusCap->render();
	glTranslatef(0.0f, 0.0f, 0.005f);
	gMinus->render();
	glTranslatef(0.0f, 0.0f, 0.005f);
	gCylCap->render();
	gMinusCap->render();
	glPopMatrix();

	if(!plus) 
		return;

// Draw Plus (90 degree rotated minus)
	glPushMatrix();
	glRotatef(90.0f, 0.0f, 0.0f, 1.0f);
	gMinusCap->render();
	glTranslatef(0.0f, 0.0f, 0.005f);
	gMinus->render();
	glTranslatef(0.0f, 0.0f, 0.005f);
	gCylCap->render();
	gMinusCap->render();
	glPopMatrix();
}

