/**************************************************************************
 *	Copyright (C) 2004-2007  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "fe_include.hpp"
#include "fe_host.hpp"
#include "fe_draw.hpp"

using namespace ICMATH;

void FeHost::render()
{
	m_Dirty = false;

	glColor3f(1.0f, 1.0f, 1.0f);
	
	Geo	*label = m_GeoCache->getGeometry("ipLabel");

	// TODO Fix this.
	if(m_Addresses.size() > 0)
		m_TexCache->getTextTexture(m_Addresses.at(0)->getAddress().c_str(), 128, 5)->bindTexture();

	glEnable(GL_TEXTURE_2D);
	glPushMatrix();
		glTranslatef(0.0f, -0.06f, -0.04f);
		glRotatef(45, 0.0f, -1.0f, 0.0f);
		label->render();
	glPopMatrix();

	if(m_Tex == 0)
	{
		m_Tex = m_TexCache->getImageTexture(m_TexID);
	}
	
	// TODO Change this
	if(m_Tex != 0)
		m_Tex->bindTexture();

	if(m_Geo == 0)
	{
		m_Geo =	m_GeoCache->getGeometry(m_GeoID.c_str());
	}
/*
	if(m_Selected)
	{
		Geo *g = m_GeoCache->getGeometry("fatSphere");
		g->render();
	}
*/
	// TODO Change this
	if(m_Geo != 0)
		m_Geo->render();
//	else
//		cerr<<"Error: Couldn't find geometry with id: "<<m_GeoID<<endl;	
	
	glDisable(GL_TEXTURE_2D);

	m_PortsNode->render();
}

void FeHost::RenderSelect()
{
	// TODO push name for selection here
	
	Geo	*label = m_GeoCache->getGeometry("ipLabel");

	glPushMatrix();
		glTranslatef(0.0f, -0.06f, -0.04f);
		glRotatef(45, 0.0f, -1.0f, 0.0f);
		label->render();
	glPopMatrix();

	if(m_Geo == 0)
	{
		m_Geo =	m_GeoCache->getGeometry(m_GeoID.c_str());
	}

	// TODO Change this
	if(m_Geo != 0)
		m_Geo->render();
	
	m_PortsNode->RenderSelect();
}

void FeHost::addPort(FePort *port)
{
	if(!port){
		cerr<<"FeHost::addPort : adding null port"<<endl;
		return;
	}

	// TODO Fix this
	m_PortsNode->addNode(port);
}

void FeHost::addHostName(string name)
{
	m_Hostnames.push_back(name);
}

void FeHost::addAddress(FeAddr *addr)
{
	if(!addr){
		cerr<<"FeHost::addAddress : adding null address"<<endl;
		return;
	}

	m_Addresses.push_back(addr);
}
void FeHost::addOSClass(FeOSClass *os)
{
	if(!os){
		cerr<<"FeHost::addAddress : adding null os class"<<endl;
		return;
	}
	m_OSClasses.push_back(os);
}

void FeHost::addOSMatch(FeOSMatch *os)
{
	if(!os){
		cerr<<"FeHost::addAddress : adding null os match"<<endl;
		return;
	}

	m_OSMatches.push_back(os);
}

FeOSClass* FeHost::getOSClass(unsigned int i) 
{ 
	if(i < m_OSClasses.size())
		return m_OSClasses.at(i);

	return 0;
}

FeOSMatch* FeHost::getOSMatch(unsigned int i) 
{ 
	if(i < m_OSMatches.size())
		return m_OSMatches.at(i);

	return 0;
}

// TODO flesh this out.
bool FeHost::handleEvent(Event *e)
{
	if(dynamic_cast<DeleteEvent*>(e)){

	}
	return FeNode::handleEvent(e);
}

wxXmlNode* FeHost::GetXMLNode()
{
	wxXmlNode* node = new wxXmlNode(0, wxXML_ELEMENT_NODE, wxT("host"));

	for(vector<FeAddr*>::iterator i = m_Addresses.begin(); i != m_Addresses.end(); ++i)
	{
		wxXmlNode* adrNode = new wxXmlNode(0, wxXML_ELEMENT_NODE, wxT("address"));
		adrNode->AddProperty(wxT("ip"), wxString::FromAscii((*i)->getAddress().c_str()));
		node->AddChild(adrNode);
	}

	for(vector<FeOSClass*>::iterator i = m_OSClasses.begin(); i != m_OSClasses.end(); ++i)
	{
		wxXmlNode* oscNode = new wxXmlNode(0, wxXML_ELEMENT_NODE, wxT("os"));
		oscNode->AddProperty(wxT("type"), wxString::FromAscii((*i)->getType().c_str()));
		oscNode->AddProperty(wxT("vendor"), wxString::FromAscii((*i)->getVendor().c_str()));
		oscNode->AddProperty(wxT("family"), wxString::FromAscii((*i)->getFamily().c_str()));
		oscNode->AddProperty(wxT("generation"), wxString::FromAscii((*i)->getGen().c_str()));
		node->AddChild(oscNode);
	}

//	portnode->AddProperty(wxT(""), new wxString());

	CollectXMLSubNodes(node);

	return node;
}

bool FeHost::handleMouseEvent(MouseEvent *me){

	if((me->getType() == ME_dclick) && (me->getTopHit() == this->m_SelName))
	{
		m_PortsNode->flipPorts();
		return true;
	}

	return FeNode::handleMouseEvent(me);
}

FeHost::FeHost() : FeNode()
{
	m_Dirty = true;
	m_Radius = 0.2f;

	m_Tex = 0;
	m_Geo = 0; 
	m_TexID = "unknown";

	m_PortsNode = new FePortsNode();
	m_Nodes.push_back(m_PortsNode);
}

FeHost::~FeHost()
{
	// TODO this doesn't seem quite right.	
	deleter<FeAddr> adDel;
	for_each(m_Addresses.begin(), m_Addresses.end(), adDel);
	m_Addresses.clear();
	
	deleter<FeOSMatch> osmDel;
	for_each(m_OSMatches.begin(), m_OSMatches.end(), osmDel);
	m_OSMatches.clear();

	deleter<FeOSClass> oscDel;
	for_each(m_OSClasses.begin(), m_OSClasses.end(), oscDel);
	m_OSClasses.clear();

	m_Hostnames.clear();

}
