/**************************************************************************
 *	Copyright (C) 2004-2007  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "fe_include.hpp"
#include "fe_hosts.hpp"
#include "fe_core.hpp"
#include "nmap_parser.hpp"

using namespace ICMATH;

void FeHostsNode::render()
{
	if(m_LabelTex == 0){
		m_LabelTex = m_TexCache->getTextTexture(m_Label.c_str(), 128, 5);
	}
	
	if(m_Geo == 0){
		m_Geo = m_GeoCache->getGeometry("ipLabel");
	}

	m_LabelTex->bindTexture();

	glEnable(GL_TEXTURE_2D);	
	m_Geo->render();
	glDisable(GL_TEXTURE_2D);

	if(m_ToolWorking)
	{
		renderProgress();
		return;
	}

	int nodeCount = (int)m_Nodes.size();
	float slice =  360.0f/(float)nodeCount;
	float	 big[2] = { 0, 0};
	
	unsigned int j = 0;

	if(nodeCount > 1)
	{
		float tmp = 0;
		for(vector<FeNode*>::iterator i = m_Nodes.begin(); i != m_Nodes.end(); ++i)
		{
			tmp = (*i)->getRadius();

			if(tmp > big[0])
				big[0] = tmp;
			else if(tmp > big[1])
				big[1] = tmp;
		}
		m_Radius = big[0] + big[1];
	}

	glTranslatef(0.0f, -0.035f, -0.005f);
	FeDraw::renderFoldWidget(!m_Open);
	glTranslatef(0.0f, -0.035f, 0.005f);

	if(m_Open)
	{
		for(vector<FeNode*>::iterator i = m_Nodes.begin(); i != m_Nodes.end(); ++i)
		{
			glColor3f(0.0f,0.0f,1.0f);
			glPushMatrix();
			glRotatef(slice*((float)j),0,1.0f,0);
			glDisable(GL_LIGHTING);
			glBegin(GL_LINES);
				glVertex3f( 0, 0, 0);
				if(m_Nodes.size() > 1)			
					glVertex3f( 0, -0.4f,m_Radius);
				else
					glVertex3f( 0, -0.4f,0.0f);
			glEnd();
			glEnable(GL_LIGHTING);

			if(m_Nodes.size() > 1)
				glTranslatef(0.0f,-0.4f,m_Radius);
			else
				glTranslatef(0.0f, -0.4f, 0.0f);
			
			(*i)->render();

			++j;

			glPopMatrix();
		}
	}
}

void FeHostsNode::RenderSelect()
{
	if(m_Geo == 0){
		m_Geo = m_GeoCache->getGeometry("ipLabel");
	}

	m_Geo->render();

	if(m_ToolWorking)
	{
		renderProgress();
		return;
	}

	int nodeCount = (int)m_Nodes.size();

	float slice =  360.0f/(float)nodeCount;
	float big[2] = { 0, 0};
	
	unsigned int j = 0;

	if(nodeCount > 1)
	{
		float tmp = 0;
		for(vector<FeNode*>::iterator i = m_Nodes.begin(); i != m_Nodes.end(); ++i)
		{
			tmp = (*i)->getRadius();

			if(tmp > big[0])
				big[0] = tmp;
			else if(tmp > big[1])
				big[1] = tmp;
		}
		m_Radius = big[0] + big[1];
	}

	glPushName(m_Nodes.size());

	glTranslatef(0.0f, -0.035f, -0.005f);
	FeDraw::renderFoldWidget(!m_Open);
	glTranslatef(0.0f, -0.035f, 0.005f);

	if(m_Open)
	{
		for(vector<FeNode*>::iterator i = m_Nodes.begin(); i != m_Nodes.end(); ++i)
		{
			glColor3f(0.0f,0.0f,1.0f);
			glPushMatrix();
			glRotatef(slice*((float)j),0,1.0f,0);

			if(m_Nodes.size() > 1)
				glTranslatef(0.0f,-0.4f,m_Radius);
			else
				glTranslatef(0.0f, -0.4f, 0.0f);
			
			glLoadName(j);
			(*i)->setSelectionName(j);

			(*i)->RenderSelect();

			++j;

			glPopMatrix();
		}
	}

	glPopName();
}

void FeHostsNode::renderProgress()
{
	if(m_ScanTex == 0){
		m_ScanTex = m_TexCache->getImageTexture("scan");
	}
	
	if(m_ScanGeo == 0){
		m_ScanGeo = m_GeoCache->getGeometry("scanner");
	}

	if(m_ScanTex != 0)
		m_ScanTex->bindTexture();

	glEnable(GL_TEXTURE_2D);
	glPushMatrix();
		glTranslatef(0.0f, -0.1f, 0.0f);
		glRotatef(m_ScanRot,0,1.0f,0);
		m_ScanGeo->render();
	glPopMatrix();
	glDisable(GL_TEXTURE_2D);

}

wxXmlNode* FeHostsNode::GetXMLNode()
{
	wxXmlNode *node = new wxXmlNode(0, wxXML_ELEMENT_NODE, wxT("hosts"));
//	node->AddProperty(wxT("name"), new wxString());
	CollectXMLSubNodes(node);
	return node;
}

bool FeHostsNode::handleEvent(Event *e){

	if(dynamic_cast<TimerFire*>(e)){

		if(m_ScanRot > 360.0f)
			m_ScanRot -= 360.0f;

		m_ScanRot += 10.0f;

		// TODO Handle scan node / file parsing here, not in the draw-node
		if(m_Tool != NULL)
		{
			if(m_Tool->checkFinished())
			{
				if(m_Tool->checkError())
				{
					m_DeleteMe = true;
				}
				else
				{
					NmapParser* nmapParser = new NmapParser(this);
#ifdef UNICODE					
					if(FeCore::parseFile(nmapParser, m_Tool->getOutputFile().ToAscii().data()))
#else
					if(FeCore::parseFile(nmapParser, m_Tool->getOutputFile().c_str()))
#endif
					{
//						m_ToolFile = m_Tool->getOutputFile();
						delete m_Tool;
						m_Tool	= 0;
						
						m_ToolWorking = false;
						
						m_Parent->updateRadius();
					}
					else
					{
						// TODO: Only do a certain number of retries and delete the node
						cerr<<"Couldn't parse file, trying again later"<<endl;	
					}
					
					delete nmapParser;
				}
			}
		}
	}

	return FeNode::handleEvent(e);
}

FeHost* FeHostsNode::getNewHost()
{
	FeHost* host = new FeHost();
	host->setParentNode(this);
	m_Nodes.push_back(host);
	return host;
}

FeHostsNode::FeHostsNode(const char *label, FeTool* tool)
		: FeNode(), m_Label(label), m_Geo(0), m_LabelTex(0), m_ScanGeo(0), 
			m_ScanTex(0), m_ScanRot(0.0f), m_Tool(tool), m_ToolWorking(false)
{
	setRadius(0.7f);
	
	if(tool == 0)
		m_ToolWorking = false;
	else
		m_ToolWorking = true;
};

