/**************************************************************************
 *	Copyright (C) 2006-2007  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330,
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "fe_include.hpp"
#include "fe_match.hpp"
#include "fe_host.hpp"

void FeMatch::addOSClass(FeOSClassRgx *os)
{
	if(!os){
		cerr<<"FeHost::addAddress : adding null os class"<<endl;
		return;
	}
	m_OSClasses.push_back(os);
}

int FeMatch::testMatch(FeHost *host)
{
	// TODO use STL here, this can be much cleaner
	// maybe overloaded == operators or FeOS self matching?
	
	// Hosts
	FeOSClass 	*osClassH 	= 0;
	FeOSMatch 	*osMatchH 	= 0;
	
	// Patterns
	FeOSClassRgx 	*osClassP 	= 0;
	FeOSMatchRgx 	*osMatchP 	= 0;	
	
	bool fitClass		= false;
	bool fitMatch		= false;
	
	if(host->getOSClassCount() > 0)
		osClassH = host->getOSClass(0);
		
	if(host->getOSMatchCount() > 0)
		osMatchH = host->getOSMatch(0);
	
	for(unsigned int i = 0; i < m_OSClasses.size(); ++i)
	{
		osClassP = m_OSClasses.at(i);
		
		if((*osClassP) == (*osClassH))
		{
			fitClass = true;
			break;
		}
	}

	for(unsigned int i = 0; i < m_OSMatches.size(); ++i)
	{
		osMatchP = m_OSMatches.at(i);
		
		if((*osMatchP) == (*osMatchH))
		{
			fitMatch = true;
			break;
		}		
	}	

	if((!fitClass) && (!fitMatch))
		return 0;
	else if ((fitClass) || (fitMatch))
		return 1;
		
	return 2;	
}

FeMatch::FeMatch()
{
	m_TextureID = "";
	m_GeometryID = "";
}

FeMatch::~FeMatch()
{
	deleter<FeOSClassRgx> cDel;
	for_each(m_OSClasses.begin(), m_OSClasses.end(), cDel);
	
	deleter<FeOSMatchRgx> mDel;
	for_each(m_OSMatches.begin(), m_OSMatches.end(), mDel);
}


void FeMatchParser::addMatch(FeMatch *m)
{
	if(m) m_Matches.push_back(m);
}

vector<FeMatch*>* FeMatchParser::findMatch(FeHost* host)
{
	if(host == 0) return 0;
	
	FeMatch*			match 	= 0;
	vector<FeMatch*>*	matches = new vector<FeMatch*>();

	if((host->getOSClassCount() == 0) && (host->getOSMatchCount() == 0))
	{
		// TODO Change this so it returns an "Unknown"
		//cout<<" FeMatch::testMatch : couldn't find any OS Classes or Matches"<<endl;
		cerr<<"Could not find any OS Classes or Matches for target, did you enable OS detection?"<<endl;
		
		match = new FeMatch();
		
		match->setTexture("unknown");
		match->setGeometry("unknown");

		matches->push_back(match);
		return matches;
	}

			
	for(unsigned int i = 0; i < m_Matches.size(); ++i)
	{
		match  = m_Matches.at(i);
		
		if(match->testMatch(host) > 0)
		{
			matches->push_back(match);
		}
	}

	return matches;
}

void FeMatchParser::startElement(	const   XMLCh* const    uri,
						const   XMLCh* const    localname,
						const   XMLCh* const    qname,
					    const   Attributes&		attributes)
{
    char* tag = XMLString::transcode(localname);
    
	if(strcmp(tag, "texture") == 0)
	{
		char* tex = XMLString::transcode(attributes.getValue(XMLString::transcode("texture_id")));
	    char* file = XMLString::transcode(attributes.getValue(XMLString::transcode("file")));
		
		Tex* tmpTex = new TexImage(tex, m_DataDir + file);

		m_TexCache->addImageTexture(tmpTex);

		XMLString::release(&file);
		XMLString::release(&tex);
	}
	else if(strcmp(tag, "match") == 0)
	{
		char* tex = XMLString::transcode(attributes.getValue(XMLString::transcode("texture_id")));
	    char* geo = XMLString::transcode(attributes.getValue(XMLString::transcode("geometry_id")));

		curMatch = new FeMatch();
		if(tex) curMatch->setTexture(tex);
		if(geo) curMatch->setGeometry(geo);

		XMLString::release(&geo);
		XMLString::release(&tex);
	}
	else if(strcmp(tag, "osclass") == 0) // TODO: This duplicates nmap_parser.cpp
	{
		char* type = XMLString::transcode(attributes.getValue(XMLString::transcode("type")));
	    char* vendor = XMLString::transcode(attributes.getValue(XMLString::transcode("vendor")));
	    char* family = XMLString::transcode(attributes.getValue(XMLString::transcode("osfamily")));
	    char* gen = XMLString::transcode(attributes.getValue(XMLString::transcode("osgen")));

		FeOSClassRgx *os = new FeOSClassRgx(); // type, vendor, family, atoi(acc));

		if(type)
			os->setType(type);
		else
			os->setType(".*");

		if(vendor) 
			os->setVendor(vendor);
		else
			os->setVendor(".*");

		if(family)
			os->setFamily(family);
		else
			os->setFamily(".*");

		if(gen)
			os->setGen(gen);
		else
			os->setGen(".*");

		curMatch->addOSClass(os);

		XMLString::release(&gen);
		XMLString::release(&family);
		XMLString::release(&vendor);
		XMLString::release(&type);
	}
	else if(strcmp(tag, "osmatch") == 0)
	{
		char* name = XMLString::transcode(attributes.getValue(XMLString::transcode("name")));
//		FeOSMatchRgx *osMatch = new FeOSMatchRgx(name);
		XMLString::release(&name);
	}

	delete[] tag;
}

void FeMatchParser::endElement( const XMLCh* const uri,
					 const XMLCh* const localname,
					 const XMLCh* const qname)
{
	char* tag = XMLString::transcode(localname);
	if(strcmp(tag, "match") == 0)
	{
		this->addMatch(curMatch);
	}
	XMLString::release(&tag);
}

void FeMatchParser::endDocument()
{

}

void FeMatchParser::fatalError(const SAXParseException&)
{

}

FeMatchParser* FeMatchParser::_instance = 0;

FeMatchParser* FeMatchParser::instance()
{
	if(_instance == 0)
		_instance = new FeMatchParser();

	return _instance;
}

FeMatchParser::FeMatchParser()
{
	curMatch = 0;
}

FeMatchParser::~FeMatchParser()
{
	deleter<FeMatch> mDel;
	for_each(m_Matches.begin(), m_Matches.end(), mDel);
}
