/**************************************************************************
 *	Copyright (C) 2004-2007  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "fe_include.hpp"
#include "fe_node.hpp"

 void FeNode::RenderBoundBox()
 {
 
 
 	
 }
 
  
bool FeNode::pushEvent(Event *e){
	bool handled = false;
	
	for(vector<FeNode*>::iterator i = m_Nodes.begin(); i != m_Nodes.end(); ++i){
		handled |= (*i)->handleEvent(e);						
	}

	return handled;
}

bool FeNode::handleEvent(Event *e){
	
	bool handled = false;

	if(dynamic_cast<DeSelectEvent*>(e)){
		m_Selected = false;
	}
	else if(dynamic_cast<MouseEvent*>(e)){
		if(handleMouseEvent((MouseEvent*)e))
			return true;
	}
	else if(dynamic_cast<UpdateRadiusEvent*>(e)){
		updateRadius();
	}

	return handled || pushEvent(e);
}

bool FeNode::handleMouseEvent(MouseEvent *me){

	bool handled = false;

	switch(me->getType())
	{
	case ME_dclick:
	case ME_click:
		if(me->wasHit())
		{
			if(this->m_SelName == me->getTopHit())
			{
				handled = true;
				m_Selected = true;
				me->decHit();

				if(me->getTopHit() == m_Nodes.size())
				{
//					cout<<"Do we have a hit?"<<endl;
					m_Open = !m_Open;
				}
			}
			else
			{
				if(me->getMod() != ME_ctrl)
				{
					m_Selected = false;
					DeSelectEvent *desel = new DeSelectEvent();
					pushEvent(desel);
					delete desel;
				}
				return true;
			}
		}
		break;

	case ME_up:
	case ME_down:
	case ME_motion:
		break;
	}

	return false;
}

void FeNode::addNode(FeNode* node)
{ 
	if(node == 0)
		return;

	node->setParentNode(this);
	m_Nodes.push_back(node); 
}

FeNode* FeNode::getParentNode()
{
	return m_Parent;
}

unsigned int FeNode::getSubNodeCount() 
{ 
	return (unsigned int)m_Nodes.size(); 
}

void FeNode::DeleteSubNodes()
{
	for(vector<FeNode*>::iterator i = m_Nodes.begin(); i != m_Nodes.end(); ++i)
	{
		if((*i)->ToDelete())
		{
			FeNode* node = (*i);
			m_Nodes.erase(i);
			delete node;
			return;
		}
	}
}

FeNode* FeNode::getNode(unsigned int node)
{
	if(node > (unsigned int)m_Nodes.size())
		return 0;

	return m_Nodes.at(node);
}

void FeNode::CollectXMLSubNodes(wxXmlNode* node)
{
	for(vector<FeNode*>::iterator i = m_Nodes.begin(); i != m_Nodes.end(); ++i){
		node->AddChild((*i)->GetXMLNode());
	}
}

void FeNode::updateRadius()
{
	unsigned int nodeCount = m_Nodes.size();
	float circ = 0.0f;
	
	switch(nodeCount)
	{
		case 1:
			m_Radius = m_Nodes.at(0)->getRadius();
		break;

		case 2:
			m_Radius = (m_Nodes.at(0)->getRadius() + m_Nodes.at(1)->getRadius()) / 2.0;
		break;

		default:
			for(vector<FeNode*>::iterator i = m_Nodes.begin(); i != m_Nodes.end(); ++i)
			{
				circ += (*i)->getRadius();
			}
			
			m_Radius = circ/ICMATH::PI;		
		break;
	}
}


FeNode::FeNode()
{
	m_Selected	= false;
	m_Open		= true;
	m_DeleteMe  = false;
	m_Frame		= 0;
	m_GeoCache	= GeometryCache::instance();
	m_TexCache	= TextureCache::instance();
	m_Geo		= 0;
	m_Tex		= 0;
}

FeNode::~FeNode()
{
	deleter<FeNode> nDel;
	for_each(m_Nodes.begin(), m_Nodes.end(), nDel);	
}
