/**************************************************************************
 *	Copyright (C) 2004-2007  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "fe_include.hpp"
#include "fe_port.hpp"
	
using namespace ICMATH;

void FePort::setService(string service) 
{
	// TODO: This should allow multiple services
	m_Service = service;
	serviceTexture = m_TexCache->getTextTexture(service.c_str(), 128, 10);
}

void FePort::setPort(const char *port)
{
	//TODO This is a bit excessive
	m_Port = atoi(port);
	portTexture = m_TexCache->getTextTexture(port, 64, 20);
}

void FePort::render()
{
	int servCount = 15;

	glPushMatrix();
		glEnable(GL_TEXTURE_2D);
		glEnable(GL_LIGHTING);
		
		glRotatef(90, 0, 0, 1);
		glTranslatef(0.03f, -0.030f, 0.075f);
		portTexture->bindTexture();

		glInterleavedArrays(GL_T2F_V3F, 0, m_PortQuad);
		glDrawArrays(GL_QUADS, 0, 4);

		if(serviceTexture)
		{
			serviceTexture->bindTexture();
			glTranslatef(0.0f, -(0.01f+((((0.07f)/5)*servCount))), 0.0f);
			glInterleavedArrays(GL_T2F_V3F, 0, m_ServiceQuad);
			glDrawArrays(GL_QUADS, 0, 4);
		}
/*		
		if(m_Selected)
		{
			cout<<"FePort::render"<<endl;
			m_GeoCache->getGeometry("fatSphere")->render();
		}
*/		
		glDisable(GL_LIGHTING);
		glDisable(GL_TEXTURE_2D);
	glPopMatrix();
}

void FePort::RenderSelect()
{
	int servCount = 15;	
	glPushMatrix();
		glRotatef(90, 0, 0, 1);
		glTranslatef(0.03f, -0.030f, 0.075f);
		glInterleavedArrays(GL_T2F_V3F, 0, m_PortQuad);
		glDrawArrays(GL_QUADS, 0, 4);

		if(serviceTexture)
		{
			glTranslatef(0.0f, -(0.01f+((((0.07f)/5)*servCount))), 0.0f);
			glInterleavedArrays(GL_T2F_V3F, 0, m_ServiceQuad);
			glDrawArrays(GL_QUADS, 0, 4);
		}
	glPopMatrix();
}

wxXmlNode* FePort::GetXMLNode()
{
	wxXmlNode *node = new wxXmlNode(0, wxXML_ELEMENT_NODE, wxT("port"));
	node->AddProperty(wxT("protocol"), wxString(_T("tcp")));
	node->AddProperty(wxT("portid"), wxString::Format(_T("%i"), m_Port));
	
	switch(this->getState())
	{
	case ps_Closed:
		node->AddProperty(wxT("state"), wxT("closed"));
		break;
		
	case ps_Open:
		node->AddProperty(wxT("state"), wxT("open"));
		break;
		
	case ps_Filtered:
		node->AddProperty(wxT("state"), wxT("filtered"));
		break;
	}

	switch(this->getProto())
	{
	case pp_TCP:
		node->AddProperty(wxT("proto"), wxT("tcp"));
		break;
		
	case pp_UDP:
		node->AddProperty(wxT("proto"), wxT("udp"));
		break;
	}
	
	return node;
}

FePort::FePort(PortProto p) 
	: FeNode(), m_Proto(p)
{
	int servCount = 15;
	
	GLfloat labels[] = {1.0f, 1.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.03f, 0.0f, 0.0f, 
						0.0f, 0.0f, 0.03f, 0.07f, 0.0f, 0.0f, 1.0f, 0.0f, 0.07f, 0.0f };		
	
	memcpy(m_PortQuad, labels, sizeof(labels));
	memcpy(m_ServiceQuad, labels, sizeof(labels));
	
	m_PortQuad[13] = m_PortQuad[18] = 0.07f;
	m_ServiceQuad[13] = m_ServiceQuad[18] = (((0.07f)/5)*servCount);

	serviceTexture = 0;
}

void FePortsNode::render()
{
	Geo *tSph =  0;

	if(m_GeoCache)
		tSph = m_GeoCache->getGeometry("tinySphere");

	if(m_vaLines == 0)
	{
		m_vaLines = new GLfloat[(int)m_Nodes.size() * 6];
		int s = 0;
		for(int i = 1; i <= (int)m_Nodes.size(); ++i)
		{
			*(m_vaLines + s + 0) = 0.0f;
			*(m_vaLines + s + 1) = 0.0f  - (0.045f)*i;
			*(m_vaLines + s + 2) = 0.0f;

			*(m_vaLines + s + 3) = 0.0f;
			*(m_vaLines + s + 4) = 0.04f - (0.045f)*i;
			*(m_vaLines + s + 5) = 0.07f;
			s += 6;
		}
	}

	glPushMatrix();
		glColor3f(0.0f, 0.0f,1.0f);
		glDisable(GL_LIGHTING);
		glBegin(GL_LINES);
			glVertex3f( 0, 0, 0);
			glVertex3f( 0, -(0.1f +(0.045f)*m_Stage),0.0f);
		glEnd();
		glTranslatef(0,-0.1f,0.0f);
	glPopMatrix();
	glPushMatrix();

	glDisable(GL_LIGHTING);
	glTranslatef(0,-0.099f,0.0f);
	glInterleavedArrays(GL_V3F, 0, m_vaLines);
	glDrawArrays(GL_LINES, 0, 2*m_Stage);
	glEnable(GL_LIGHTING);

	unsigned int j = 0;

	for(vector<FeNode*>::iterator i = m_Nodes.begin(); i != m_Nodes.end(); ++i)
	{
		if(j >= m_Stage)
			break;

		glTranslatef(0,-0.045f,0.0f);

		(*i)->render();
		++j;
	}

	glPopMatrix();
	glPushMatrix();
	glTranslatef(0,-0.099f,0.0f);
	glColor3f(1.0f,1.0f,1.0f);

	if(tSph != 0)
	{
		glInterleavedArrays(tSph->getFormat(), 0, tSph->data);
	
		for(unsigned int i = 0; i < m_Stage; ++i)
		{
			glTranslatef(0,-0.045f,0.0f);
			glDrawArrays(tSph->getMode(), 0, tSph->getCount());
		}
	}
	glPopMatrix();
}

void FePortsNode::RenderSelect()
{
	Geo *tSph =  0;

	if(m_GeoCache)
		tSph = m_GeoCache->getGeometry("tinySphere");

	glPushMatrix();
		glPushName(0);
		glTranslatef(0,-0.099f,0.0f);
	
		unsigned int j = 0;
		for(vector<FeNode*>::iterator i = m_Nodes.begin(); i != m_Nodes.end(); ++i)
		{
			if(j >= m_Stage)
				break;
	
			glTranslatef(0,-0.045f,0.0f);
	
			glLoadName(j);
			(*i)->setSelectionName(j);
	
			(*i)->RenderSelect();
			++j;
		}
	
		glPopName();

	glPopMatrix();
	glPushMatrix();
		glTranslatef(0,-0.099f,0.0f);
		glColor3f(1.0f,1.0f,1.0f);
	
		if(tSph != 0)
		{
			glInterleavedArrays(tSph->getFormat(), 0, tSph->data);
		
			for(unsigned int i = 0; i < m_Stage; ++i)
			{
				glTranslatef(0,-0.045f,0.0f);
				glDrawArrays(tSph->getMode(), 0, tSph->getCount());
			}
		}
	glPopMatrix();
}

wxXmlNode* FePortsNode::GetXMLNode()
{
	wxXmlNode *node = new wxXmlNode(0, wxXML_ELEMENT_NODE, wxT("ports"));

	CollectXMLSubNodes(node);

	return node;
}

bool FePortsNode::handleEvent(Event *e){

	if(dynamic_cast<TimerFire*>(e)){
		if(m_MovDone)
			return false;

		if(m_PortsDown)
		{
			if(m_Stage > 0)
				--m_Stage;
			else
				m_MovDone = true;
		}
		else
		{
			if(m_Stage < (unsigned int)m_Nodes.size())
				++m_Stage;
			else
				m_MovDone = true;
		}
		return true;
	}
	return FeNode::handleEvent(e);
}

void FePortsNode::UpdateBoundingBox()
{
	
	
}

FePortsNode::FePortsNode() : FeNode()
{
	m_vaLines = 0;
	m_Stage = 0;
	m_MovDone = false;
	m_PortsDown = false;
}

FePortsNode::~FePortsNode()
{
//	deleter<FePort> pDel;
//	for_each(m_Ports.begin(), m_Ports.end(), pDel);
//	m_Ports.clear();

	if(m_vaLines)
		delete[] m_vaLines;
}

