/**************************************************************************
 *	Copyright (C) 2004-2007  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "fe_include.hpp"
#include "fe_root_node.hpp"

void FeRootNode::render()
{
	// TODO move this somewhere else
	DeleteSubNodes();
	
	//TODO Fix this (i.e. don't "load" geometry everytime
	Geo *g = m_GeoCache->getGeometry("rootSphere");

	if(g == 0){
		cerr<<"Can't find rootSphere geometry"<<endl;
		return;
	}

	if(m_RootTex == 0){
		m_RootTex = m_TexCache->getImageTexture("rootTex");

		if(m_RootTex == 0){
			cerr<<"Can't find Root Node Texture"<<endl;
			return;
		}
	}

	m_RootTex->bindTexture();

	glPushMatrix();
		glRotatef(m_Config->getRootTilt(), 1.0f, 0.0f, 0.0f);
		glEnable(GL_LIGHTING);
		glEnable(GL_TEXTURE_2D);
		glColor3f(1.0f, 1.0f, 1.0f);

		g->render();

		glDisable(GL_TEXTURE_2D);
		glDisable(GL_LIGHTING);
		glColor3f(0.0f, 0.0f, 0.5f);
		glBegin(GL_LINES);
			glVertex3f( 0, 0.2f, 0);
			glVertex3f( 0, -0.2f,0.0f);
		glEnd();
	glPopMatrix();


	int nodeCount = (int)m_Nodes.size();
	
	if(nodeCount <= 0) 
		return;

	int j = 0;
	float slice =  360.0f/(float)nodeCount;

	for(vector<FeNode*>::iterator i = m_Nodes.begin(); i != m_Nodes.end(); ++i)
	{
		glColor3f(0.0f,0.0f,1.0f);
		glPushMatrix();
		glRotatef(slice*((float)j),0,1.0f,0);
		glDisable(GL_LIGHTING);
		glBegin(GL_LINES);
			glVertex3f( 0, 0, 0);
			if(m_Nodes.size() > 1)			
				glVertex3f( 0, -0.4f,m_Radius);
			else
				glVertex3f( 0, -0.4f,0.0f);
		glEnd();
		glEnable(GL_LIGHTING);
		
		if(m_Nodes.size() > 1)
			glTranslatef(0.0f,-0.4f,m_Radius);
		else
			glTranslatef(0.0f, -0.4f, 0.0f);
		
		(*i)->render();
		
		glPopMatrix();
		++j;
	}

}

void FeRootNode::RenderSelect()
{
	//TODO Fix this (i.e. don't "load" geometry everytime
	Geo *g = m_GeoCache->getGeometry("rootSphere");

	if(g == 0){
		cerr<<"Can't find rootSphere geometry"<<endl;
		return;
	}

	glPushMatrix();
		glRotatef(m_Config->getRootTilt(), 1.0f, 0.0f, 0.0f);
		glColor3f(1.0f, 1.0f, 1.0f);

		g->render();

		glColor3f(0.0f, 0.0f, 0.5f);
	glPopMatrix();


	int nodeCount = (int)m_Nodes.size();
	
	if(nodeCount <= 0) 
		return;

	int j = 0;
	float slice =  360.0f/(float)nodeCount;

	glPushName(0);

	glColor3f(0.0f,0.0f,1.0f);
	for(vector<FeNode*>::iterator i = m_Nodes.begin(); i != m_Nodes.end(); ++i)
	{
		glPushMatrix();
			glRotatef(slice*((float)j),0,1.0f,0);
	
			if(m_Nodes.size() > 1)
				glTranslatef(0.0f,-0.4f,m_Radius);
			else
				glTranslatef(0.0f, -0.4f, 0.0f);
			
			glLoadName(j);
			(*i)->setSelectionName(j); // TODO ? 
			(*i)->RenderSelect();
		glPopMatrix();
		++j;
	}

	glPopName();
}

wxXmlNode* FeRootNode::GetXMLNode()
{
	wxXmlNode *node = new wxXmlNode(0, wxXML_ELEMENT_NODE, wxT("fe3d"));
//	node->AddProperty(wxT("version"), this->m_ExecPath);

	CollectXMLSubNodes(node);
	return node;
}

void FeRootNode::clear()
{
	for(vector<FeNode*>::iterator i = m_Nodes.begin(); i != m_Nodes.end(); ++i)
	{
		delete (*i);
	}
	
	m_Nodes.clear();
}

bool FeRootNode::handleEvent(Event *e)
{
//	DeleteSubNodes();
	return FeNode::handleEvent(e);
}

FeRootNode::FeRootNode(FeConfig *config) 
	: FeNode(), m_Config(config)
{
	m_Radius	= 0.1f;
	m_RootTex	= 0;
	m_SelName = 0;
}
