/**************************************************************************
 *	Copyright (C) 2005-2007  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "fe_include.hpp"
#include "fe_tools.hpp"

using namespace std;

BEGIN_EVENT_TABLE(FeTool, wxEvtHandler)
	EVT_END_PROCESS(wxID_ANY, FeTool::OnToolEnd)
END_EVENT_TABLE()


bool FeTool::launch(vector<wxString>* args)
{
	if(args == NULL)
	{
		cerr<<"FeTool::launch : Can't launch nmap with null arguments"<<endl;
		m_Finished = true;
		m_Error = true;
		return false;
	}
	
	m_Finished = false;	

	if(m_OutputArg.length() > 0){
		args->push_back(m_OutputArg);
	}

	if(m_OutputFile.length() > 0){
		args->push_back(m_OutputFile);
	}

	for(unsigned int i = 0; i < args->size(); ++i)
	{
		wxString arg(args->at(i).c_str(), wxConvUTF8);
		m_Cmd += arg + wxT(" ");
	}

//	cmd = wxString::FromAscii(this->m_Path.c_str()) + wxT(" ") + cmd;
	m_Cmd = m_Path + wxT(" ") + m_Cmd;
	cerr<<" Launching : "<<m_Cmd<<endl;

	m_Process = new wxProcess(this);
	m_Process->Redirect();

	long proc_code = ::wxExecute(m_Cmd, wxEXEC_ASYNC, m_Process);
	
	m_ErrorStream = m_Process->GetErrorStream();
	m_OutputStream = m_Process->GetInputStream();

//	if(m_ErrorStream == 0)
///		cout<<"ErrorStream is null"<<endl;

//	if(m_OutputStream == 0)
//		cout<<"OutputStream is null"<<endl;
	
	
	if(proc_code == 0)
		return false;
	
	return true;
}

bool FeTool::checkFinished()
{
	return m_Finished;
}

bool FeTool::checkError()
{
	return m_Error;
}

bool FeTool::outFileReady()
{
	return true;
}


void FeTool::OnToolEnd(wxProcessEvent& event)
{
	wxString errorString;
	wxString outputString;
	
	// TODO Remove blank lines
	if(m_ErrorStream != 0)
	{
		wxTextInputStream errOut(*m_ErrorStream);
		
		for(;;)
		{
			wxString line = errOut.ReadLine();
			if ( m_ErrorStream->Eof() && line.empty() )
                break;
                
            if(!line.empty())
				errorString = errorString + wxT("\n") + line;
		}
	}
	
	if(m_OutputStream != 0)
	{
		wxTextInputStream textOut(*m_OutputStream);

		for(;;)
		{
			wxString line = textOut.ReadLine();
			if ( m_OutputStream->Eof() && line.empty() )
                break;
                
            if(!line.empty())                
				outputString = outputString + wxT("\n") + line;				
		}
	}
	
//	cout<<"Output Stream: \n"<<outputString<<endl;
//	cout<<"Error Stream: \n"<<errorString<<endl;
	
	if(event.GetExitCode() != 0)
	{
//		cout<<"Process finished with code: "<<event.GetExitCode()<<endl;
	
		::wxMessageBox(_("There was a problem running the external tool command:\n\n")
					+ m_Cmd + _("\n\nThe tool error output was:\n") +
					_("-------------------\n")  + errorString + _("\n-------------------\n") +
					_("\nPlease try running the command directly from a terminal"), 
					_("Tool Error"), wxOK | wxICON_ERROR, m_Parent);		
	
		m_Error = true;	
	}
	m_Finished = true;
}

wxWindow* FeTool::m_Parent = 0;

FeTool::FeTool(wxString name, wxString path, wxString exec)
		: m_Name(name), m_Path(path), m_Exec(exec), m_Finished(false), m_Error(false)
{


}

