/**************************************************************************
 *	Copyright (C) 2004-2007  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "fe_include.hpp"
#include "geo_cache.hpp"

void Geo::render()
{
	glPushMatrix();
		glRotatef(rotate[0],rotate[1], rotate[2], rotate[3]);
		glTranslatef(trans[0], trans[1], trans[2]);
		glInterleavedArrays(m_Format, 0, data);
		glDrawArrays(m_Mode, 0, m_Count);
	glPopMatrix();
}

void Geo::setTransform(float x, float y, float z)
{
	trans[0] = x;
	trans[1] = y;
	trans[2] = z;
}

void Geo::setRotation(float ang, float x, float y, float z)
{
	rotate[0] = ang;
	rotate[1] = x;
	rotate[2] = y;
	rotate[3] = z;
}

Geo::Geo(GLenum f, GLenum m, int c)
{
	m_Count		= c;
	m_Mode		= m;
	m_Format	= f;
	data		= 0;

	switch(f)
	{
	case GL_T2F_N3F_V3F:
		data = new float[8*c];
		break;
	case GL_N3F_V3F:
		data = new float[6*c];
		break;
	case GL_V3F:
		data = new float[3*c];
		break;
	}

	for(int i = 0; i < 3; ++i)
		trans[i] = 0.0f;
		
	for(int i = 0; i < 4; ++i)
		rotate[i] = 0.0f;
}

Geo::~Geo()
{
	if(data){
		delete[] data;
		data = 0;
	}
}

void GeometryCache::addGeometry(std::string id, Geo *g)
{
	m_GeoCache[id] = g;
}

Geo* GeometryCache::getGeometry(const char* id)//std::string id)
{
	std::string geo_id = id;
	return m_GeoCache[geo_id];
}

GeometryCache* GeometryCache::_instance = 0;

GeometryCache* GeometryCache::instance()
{
	if(_instance == 0)
		_instance = new GeometryCache();

	return _instance;
}

GeometryCache::GeometryCache()
{

}

GeometryCache::~GeometryCache()
{
	Geo *tmp = 0;
	while(!m_GeoCache.empty())
	{
		tmp = m_GeoCache.begin()->second;
		m_GeoCache.erase(m_GeoCache.begin());
		delete tmp;
	}
	_instance = 0;
}
