/**************************************************************************
 *	Copyright (C) 2004-2007  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#if _MSC_VER > 1000
#pragma once
#endif

#ifndef H_FE3D_FE_CORE
#define H_FE3D_FE_CORE

#include "fe_include.hpp"
#include "gl_util.hpp"

#include "fe_host.hpp"
#include "fe_hosts.hpp"
#include "fe_config.hpp"
#include "fe_match.hpp"
#include "fe_draw.hpp"
#include "fe_tools.hpp"
#include "fe_root_node.hpp"
#include "geo_cache.hpp"
#include "tex_cache.hpp"

class FE3D_DECL FeCore
{	

public: // Initializations
	bool init();
	bool initGL();
	
public: // Scan Processing
	bool loadScan(const char* file, FeHostsNode *node);
	FeTool *spawnNmap(vector<wxString>* command, wxString label);
	void checkNmap();

public: // 
	bool LoadFile(wxString file);
	bool SaveFile(wxString file);

public: // Rendering
	void render(bool select);
	void renderSelect();

	bool tick() { 
		TimerFire *tf = new TimerFire();
		bool rv = m_RootNode->pushEvent(tf); 
		delete tf;
		return rv;
	}

public: // Selection
	void enablePicking(unsigned int x, unsigned int y);
	bool disablePicking();
	int processHits(GLuint hits, GLuint buffer[], GLuint *ns);

public: // Settings
	static bool parseFile(DefaultHandler* handler, string file);
	string getConfigFile(string file);

public: // Getters
	FeConfig* getConfig() { return m_Config; }
	FeRootNode* getRootNode() { return m_RootNode; }

public: // Misc
	int screenShot(void);
	void updateRadius();
	bool pushEvent(Event *e);
	void clearScene();

	void setWindowSize(int w, int h);

	void SetParentWindow(wxWindow* parent) { m_ParentWindow = parent; }

public: // Construction / Destruction
	FeCore();
	~FeCore();

private: // Data
    FeConfig* 			m_Config;
	FeMatchParser*		m_Matcher;
	GeometryCache* 		m_GeoCache;
	TextureCache* 		m_TexCache;
	
	FeRootNode*			m_RootNode;

	wxWindow*			m_ParentWindow;

	int m_Width;
	int m_Height;
	
public: // Data members
	GLfloat	rtri;
	GLfloat rotSpeed;
	GLfloat x_Rot;
	GLfloat y_Rot;
	GLfloat y_Trans;
	GLfloat	z_Trans;

	bool m_Dirty;

public: //Temp, State variables / Compat only, needs to be changed and removed

	// User states
	bool mouseDownLeft;
	bool mouseDownRight;

	bool ctrlDown;
	bool shiftDown;

	bool underSelection;			// We have just selected something
	static const int SELECT_BUFSIZE = 100;
	GLuint selectBuf[SELECT_BUFSIZE];

	unsigned int prevHits[15];		// Keep Track of the last location clicked
	unsigned int prevHitCount;
	int lastClick;					// Last click ticks
	NodeSelect	m_NodeSelect;

	int prevX;
	int prevY;

	unsigned int topHitName;
};

#endif // H_FE3D_FE_CORE
