/**************************************************************************
 *	Copyright (C) 2004  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#if _MSC_VER > 1000
#pragma once
#endif

#ifndef H_FE3D_EVENTS
#define H_FE3D_EVENTS

enum ME_Types {
	ME_motion,
	ME_up,
	ME_down,
	ME_click,
	ME_dclick,
};

enum ME_Mod {
	ME_none,
	ME_ctrl,
	ME_alt,
	ME_shift
};

class FE3D_DECL Event {
public:
	virtual ~Event(){};
};

class FE3D_DECL UpdateRadiusEvent : public Event {
public:
	virtual ~UpdateRadiusEvent(){};
};

class FE3D_DECL TimerFire : public Event {
public:
	virtual ~TimerFire(){};
};

class FE3D_DECL DeSelectEvent : public Event {
public:
	virtual ~DeSelectEvent(){};
};

class FE3D_DECL DeleteEvent : public Event {

public:
	DeleteEvent(NodeSelect s) { m_NodeSelect = s; }
	virtual ~DeleteEvent(){}

public:
	NodeSelect	m_NodeSelect;

};

class FE3D_DECL MouseEvent : public Event {

public:
	unsigned int getX() { return m_X; }
	unsigned int getY() { return m_Y; }

	unsigned char getButton() { return m_Button; }
	ME_Types getType() { return m_Type; }
	ME_Mod getMod() { return m_Modifier; }

	bool wasHit() { return m_HasHits; }

	void setHitData(int count, unsigned int *hits)
	{
		m_NodeSelect.m_LocalHitCount = m_NodeSelect.m_HitCount = count;
		m_NodeSelect.m_LocalHits = m_NodeSelect.m_Hits = hits;
		m_HasHits = true;
	}

	void decHit()
	{
		if(m_NodeSelect.m_LocalHits == 0)
			return;

		--m_NodeSelect.m_LocalHitCount;
		m_NodeSelect.m_LocalHits = &(m_NodeSelect.m_LocalHits[1]);
	}

	int getHitCount() { return m_NodeSelect.m_LocalHitCount; }

	unsigned int getTopHit()
	{
		if(m_NodeSelect.m_LocalHits == 0)
			return 0;

		return m_NodeSelect.m_LocalHits[0];
	}

public: // Construction / Destruction
	MouseEvent(unsigned int x, unsigned int y, unsigned char button, ME_Types type, ME_Mod mod = ME_none) :
	  m_X(x), m_Y(y), m_Button(button), m_Type(type), m_Modifier(mod)
	{
		m_NodeSelect.m_Hits = m_NodeSelect.m_LocalHits = 0;
		m_HasHits = false;
	}
	
	virtual ~MouseEvent(){};

private:
	unsigned	int m_X;
	unsigned	int m_Y;
	unsigned	char m_Button;
	ME_Types	m_Type;
	ME_Mod		m_Modifier;
	bool		m_HasHits;

	NodeSelect	m_NodeSelect;

};

#endif // H_FE3D_EVENTS
