/**************************************************************************
 *	Copyright (C) 2004-2007  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#if _MSC_VER > 1000
#pragma once
#endif

#ifndef H_FE3D_FE_HOST
#define H_FE3D_FE_HOST

#include "fe_include.hpp"
#include "fe_port.hpp"
#include "fe_os.hpp"
#include "fe_node.hpp"

using namespace std;

class FE3D_DECL FeAddr
{
public: // Setter / Getters
	AddrType getAddrType() { return m_Type; };
	string getAddress() { return m_Address; };

public:	// Construction / Destruction
	FeAddr(AddrType type, string addr)
		: m_Type(type), m_Address(addr)
	{ }

private: // Data
	AddrType m_Type;
	string	m_Address;
};	

class FE3D_DECL FeHost : public FeNode
{
public: // Rendering functions
	virtual void render();
	virtual void RenderSelect();

public: // Event Handling
	virtual bool handleEvent(Event *e);
	virtual bool handleMouseEvent(MouseEvent *me);

public: // Creation Methods
	void addPort(FePort *port);
	void addHostName(string name);
	void addAddress(FeAddr *addr);
	void addOSClass(FeOSClass *os);
	void addOSMatch(FeOSMatch *os);
	
public: // Data access methods
	FeOSClass* getOSClass(unsigned int i);
	FeOSMatch* getOSMatch(unsigned int i);

	unsigned int getOSClassCount() { return (unsigned int)m_OSClasses.size(); };
	unsigned int getOSMatchCount() { return (unsigned int)m_OSMatches.size(); };

public: // File handling
	virtual wxXmlNode* GetXMLNode();

public:	// Construction / Destruction
	FeHost();
	virtual ~FeHost();

protected: // Data
	FePortsNode			*m_PortsNode;	// Node containing ports

	vector<FeAddr*>		m_Addresses;	// Addresses
	vector<string>		m_Hostnames;	// Hostnames
	vector<FeOSClass*>	m_OSClasses;	// OS Classes
	vector<FeOSMatch*>	m_OSMatches;	// OS Matches

	bool				m_Dirty;
};	

#endif // H_FE3D_FE_HOST
