/**************************************************************************
 *	Copyright (C) 2004-2007 iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#if _MSC_VER > 1000
#pragma once
#endif

#ifndef H_FE3D_MATCH
#define H_FE3D_MATCH

#include "fe_include.hpp"
#include "fe_xml_util.hpp"
#include "fe_os_rgx.hpp"
#include "fe_host.hpp"


using namespace std;
XERCES_CPP_NAMESPACE_USE

class FE3D_DECL FeMatch
{
public: // Creation
	void addOSClass(FeOSClassRgx *os);
	void addOSMatch(FeOSMatchRgx *os);

public: // Access
	void setTexture(string s) { m_TextureID = s; };
	string getTexture() { return m_TextureID; };

	void setGeometry(string g) { m_GeometryID = g; };
	string getGeometry() { return m_GeometryID; };

public: // Searching
	int testMatch(FeHost *host); // 0 - not a match, 1 - partial match, 2 - full match

public: // initialization
	static void destroy();

public: // Construction / Destruction
	FeMatch();
public:
	~FeMatch();

private: // Data
	vector<FeOSClassRgx*>	m_OSClasses;	// OS Classes
	vector<FeOSMatchRgx*>	m_OSMatches;	// OS Matches
	
	string	m_TextureID;
	string	m_GeometryID;

};

class FE3D_DECL FeMatchParser : public DefaultHandler
{
	
public:
	void addMatch(FeMatch *m); 
	vector<FeMatch*>* findMatch(FeHost* host);

public: // SAX2 Functionality
    void startElement(	const   XMLCh* const    uri,
						const   XMLCh* const    localname,
						const   XMLCh* const    qname,
					    const   Attributes&		attributes);

	void endElement( const XMLCh* const uri,
					 const XMLCh* const localname,
					 const XMLCh* const qname);

	void endDocument();
    void fatalError(const SAXParseException&);

public: // Setter / Getters
	void setDataDir(const char* dataDir) { m_DataDir = dataDir; }

public: // Init stuff
	static FeMatchParser* instance();

	void setTextureCache(TextureCache* tc) { m_TexCache = tc; }


private: // Construction / Destruction
	FeMatchParser();
public: 
	~FeMatchParser();

private: // Data
	static FeMatchParser* _instance;

	FeMatch*			curMatch;
	vector<FeMatch*> 	m_Matches;
	TextureCache*		m_TexCache;
	string				m_DataDir;
};
#endif // H_FE3D_MATCH

