/**************************************************************************
 *	Copyright (C) 2004-2007  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#if _MSC_VER > 1000
#pragma once
#endif

#ifndef H_FE3D_NODE
#define H_FE3D_NODE

#include "fe_include.hpp"
#include "fe_events.hpp"
#include "geo_cache.hpp"
#include "tex_cache.hpp"
#include "gl_util.hpp"

class FE3D_DECL FeNode
{
public: // Rendering
	virtual void render() = 0;
	virtual void RenderSelect() = 0;
	virtual void RenderBoundBox();

public: // Event handling
	virtual bool pushEvent(Event *e);
	virtual bool handleEvent(Event *e);
	virtual bool handleMouseEvent(MouseEvent *me);

public: // Node Handling
	void addNode(FeNode* node);
	FeNode* getNode(unsigned int node);
	void setParentNode(FeNode* parent) { m_Parent = parent; }
	FeNode* getParentNode();
	unsigned int getSubNodeCount();
	void DeleteSubNodes();

public: // Setter / Getters
	float getRadius() { return m_Radius; };
	void setRadius(float rad) { m_Radius = rad; };

	void setSelected(bool s) { m_Selected = s; };
	bool isSelected() { return m_Selected; };

	unsigned int getSelectionName() { return m_SelName; }
	void setSelectionName(unsigned int n) { m_SelName = n; }

	void setGeometry(Geo *g) { m_Geo = g; }
	void setTexture(Tex *t) { m_Tex = t; }	

	void setGeometryID(string g) { m_GeoID = g; }
	void setTextureID(string t) { m_TexID = t; }

	bool ToDelete() { return m_DeleteMe; }

public: // File handling
	virtual wxXmlNode* GetXMLNode() = 0;
	void CollectXMLSubNodes(wxXmlNode* node);

public: // Misc
	virtual void updateRadius();
	bool toDelete() { return m_DeleteMe; }

public:	// Construction/Destruction
	FeNode();
	virtual ~FeNode();

protected: // Data
	FeNode*				m_Parent;

	float				m_Radius;
	vector<FeNode*> 	m_Nodes;
		
	Geo*				m_Geo;
	Tex*				m_Tex;
	string				m_GeoID;
	string				m_TexID;

	GeometryCache*		m_GeoCache;
	TextureCache*		m_TexCache;

	bool				m_Open;		// Are the sub-nodes collapsed?
	bool				m_Selected;
	
	bool				m_DeleteMe; // A flag indicating this node should be deleted and erased

	unsigned int		m_SelName;		// Selection Mode 
	unsigned int		m_Frame;		// Frame number

	float				m_Pos[3];
};	

#endif // H_FE3D_NODE



