/**************************************************************************
 *	Copyright (C) 2004-2007  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#if _MSC_VER > 1000
#pragma once
#endif

#ifndef H_FE3D_FE_PORT
#define H_FE3D_FE_PORT

#include "fe_include.hpp"
#include "fe_defines.hpp"
#include "fe_node.hpp"
#include "gl_util.hpp"
//#include "geo_cache.hpp"
#include "tex_cache.hpp"

using namespace std;

class FE3D_DECL FePort :  public FeNode
{
public: // Rendering
	virtual void render();
	virtual void RenderSelect();

public: // Setters / Getters
	void setPort(const char *p);
	int getPort()	{ return m_Port; };

	void setService(string s);
	string getService() { return m_Service; };

	void setState(PortState s) { m_State = s; };
	PortState getState() { return m_State; };

	void setProto(PortProto p) { m_Proto = p; };
	PortProto getProto() { return m_Proto; };

public: // File handling
	virtual wxXmlNode* GetXMLNode();

public:	// Construction / Destruction
	FePort(PortProto p);
	
private: // Data
	int			m_Port;
	string		m_Service;
	PortState	m_State;
	PortProto	m_Proto;

private: // OpenGL data
	Tex*		portTexture;
	Tex*		serviceTexture;
	
	GLfloat		m_PortQuad[20];
	GLfloat		m_ServiceQuad[20];
};

class FE3D_DECL FePortsNode : public FeNode
{
public: // Rendering
	virtual void render();
	virtual void RenderSelect();

public: // Event Handling
	virtual bool handleEvent(Event *e);

public: // File handling
	virtual wxXmlNode* GetXMLNode();

public: // Misc
	void hidePorts() { m_PortsDown = false; }
	void showPorts() { m_PortsDown = false; }

	void flipPorts() {
		m_PortsDown = !m_PortsDown;
		m_MovDone = false;
	}
	
protected: // 
	virtual void UpdateBoundingBox();
	
public: // Construction / Destruction
	FePortsNode();
	virtual ~FePortsNode();

private: // Data
	unsigned int	m_Stage;
	bool			m_PortsDown;
	bool			m_MovDone;
	float*			m_vaLines;
};

#endif // H_FE3D_FE_PORT

