/**************************************************************************
 *	Copyright (C) 2004-2007  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#if _MSC_VER > 1000
#pragma once
#endif

#ifndef H_FE3D_TEX_CACHE
#define H_FE3D_TEX_CACHE

#include "fe_include.hpp"
#include "gl_util.hpp"

using namespace std;

class FE3D_DECL Tex
{

public: // Usage
	virtual void bindTexture();

public: // Setter / Getters
	string getId() { return texId; };
	GLuint getTexObj() { return texObj; };
	void invalidate() { loaded = false; };
	virtual void load() = 0; // Should this be protected?

public: // Construction / Destruction
	Tex(string id)
		: texId(id), texObj(0), loaded(false), error(false)
	{ };

	virtual ~Tex();
	
protected: // Data
	string		texId;
	GLuint		texObj;
	bool		loaded;
	bool		error;

};

class FE3D_DECL TexImage : public Tex
{
protected:
	virtual void load();

public: // Construction / Destruction
	TexImage(string id, string f)
		: Tex(id), file(f)
	{ };
//	virtual ~TexImage();

protected: // Data
	string		file;
};

class FE3D_DECL TexText : public Tex
{
protected:
	virtual void load();

public: // Setter / Getters

	void setFont(wxFont* f) { font = f; }

public: // Construction / Destruction
	TexText(string id, string text, int sz)
		: Tex(id), text(text), size(sz)
	{ };
//	virtual ~TexText();
	
protected: // Data
	string		text;
	wxFont*		font;
	int			size;
};

/**
 * \ingroup Miscellaneous Files
 *
 * \version 0.9
 * first version
 *
 * \date 13 Aug 2006
 *
 * \author Stou Sandalski
 *
 * \par license GPL
 * 
 * This class is the texture loader/unloader and store for textures.
 * it can be used in 'load on demand' or 'load all at once' modes.
 *
 */
class FE3D_DECL TextureCache
{
public: // Texture Management Functions
	void addImageTexture(Tex* t);
	Tex* getImageTexture(string id);

	void addTextTexture(Tex* t);
	Tex* getTextTexture(const char* ps, unsigned int sz, unsigned int pixels);

	void invalidateTextures();

private: // Font Management
	wxFont* getFont(unsigned int pixelSize);

public: // Initialization
	static TextureCache* instance();

private: // Construction / Destruction
	TextureCache();
public:
	~TextureCache();

private: // Data
	static TextureCache* _instance;

	// TODO Combine these into one list
	map<string, Tex*>	m_textList;
	map<string, Tex*>	m_imgList;
	map<unsigned int,wxFont*>	m_fontList;
	//TODO Should this be here?
	//string			m_FontFile;
};

#endif // H_FE3D_TEX_CACHE
