/**************************************************************************
 *	Copyright (C) 2004-2007  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "fe_include.hpp"
#include "tex_cache.hpp"

using namespace std;

void Tex::bindTexture()
{
	if(!loaded)
	{
		this->load();
	}
	glBindTexture(GL_TEXTURE_2D, texObj);
}

Tex::~Tex()
{
	glDeleteTextures(1, &texObj);	
}

void TexImage::load()
{
	if(error)
		return;

	wxString str(file.c_str(), wxConvUTF8);

	wxImage image(str);

	// TODO this get stuck in a loop
	// Fix it? We might be using osgDB for this stuff anyway

	if(!image.IsOk())
	{
		cerr<<"TexImage::load(): Error loading image: "<<str<<endl;
		error = true;
		return;
	}
		
	glGenTextures(1, &texObj);

	glBindTexture(GL_TEXTURE_2D, texObj);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,GL_LINEAR);

	glTexImage2D(GL_TEXTURE_2D, 0, 3, image.GetWidth(), image.GetHeight(), 0, GL_RGB, GL_UNSIGNED_BYTE, image.GetData());
	loaded = true;
}

void TexText::load()
{
	wxMemoryDC	memoryDC;
	wxBitmap 	textBmp = wxBitmap(128,32,-1);
	
	if(!textBmp.Ok()){
		cerr<<"NULL BMP"<<endl;
		return;
	}
	
	wxString	str(text.c_str(), wxConvUTF8);
//	wxFont		font(10, wxFONTFAMILY_DEFAULT, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_BOLD);

	if(this->font != 0)
	{
		if(!font->Ok())
		{
			cerr<<"TexText::load() : We don't have a font Whats going on?"<<endl;
			delete font;
			font = 0;
		}
	}
	memoryDC.SelectObjectAsSource(textBmp);
	
	if(!memoryDC.Ok()){
		cerr<<"TexText::load() : memoryDC couldn't Select Bitmap"<<endl;
		return;
	}
	
	memoryDC.SetBackground(*wxWHITE_BRUSH);
	memoryDC.Clear();

	if(font != 0)
		memoryDC.SetFont(*this->font);
	else
		memoryDC.SetFont(*wxNORMAL_FONT);
	
	if(!memoryDC.Ok()){
		cerr<<"TexText::load() : memoryDC couldn't Set Font"<<endl;
		return;
	}
	
	wxSize size = memoryDC.GetTextExtent(str);

	int x = (128 - size.GetWidth()) / 2;
	int y = (32 - size.GetHeight()) / 2;

	// TODO Adjust this.
	if (x < 0){
		cerr<<"ERROR x < 0 : "<<size.GetWidth()<<endl;
		x = 0;
	}

	if (y < 0){
		cerr<<"ERROR y < 0:w"<< size.GetHeight()<<endl;
		y = 0;
	}
	
	memoryDC.DrawText(str, x, y);
	
	if(!memoryDC.Ok()){
		cerr<<"TexText::load() : memoryDC not OK"<<endl;
		return;
	}

	memoryDC.SelectObjectAsSource(wxNullBitmap);

	wxImage img = textBmp.ConvertToImage();

	glGenTextures(1, &texObj);
	glBindTexture(GL_TEXTURE_2D, texObj);

	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); // scale linearly when image smaller than texture
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, img.GetWidth(), img.GetHeight(), 0, GL_RGB, GL_UNSIGNED_BYTE, img.GetData());
	
	if(glGetError() != GL_NO_ERROR)
	{
		cout<<"OpenGL Error"<<endl;
	}

	loaded = true;
}

void TextureCache::addImageTexture(Tex* t)
{
	if(t == 0)
	{
		cerr<<"TextureCache::addImageTexture : Can't load NULL Tex object"<<endl;
		return;
	}
	m_imgList[t->getId()] = t;
}

Tex* TextureCache::getImageTexture(string id)
{
	if(m_imgList.size() > 0)
	{
		return m_imgList[id];
	}
	else
	{
		cerr<<"TextureCache::getImageTexture: Couldn't find texture: "<<id<<endl;
		return 0;
	}
}

void TextureCache::addTextTexture(Tex* t)
{
	if(t == 0)
	{
		cerr<<"TextureCache::addTextTexture : -> Can't load NULL Tex object"<<endl;
		return;
	}
	m_textList[t->getId()] = t;
}

Tex* TextureCache::getTextTexture(const char* ps, unsigned int sz, unsigned int pixels)
{
	map<string,Tex*>::iterator element = m_textList.find(ps);
	if(element == m_textList.end())
	{
		TexText *txt = new TexText(ps, ps, sz);

		txt->setFont(getFont(pixels));

		addTextTexture(txt);
		element = m_textList.find(ps);

		if(element == m_textList.end())
			cerr<<"TextureCache::getTextTexture: ERROR"<<endl;
	}

	return m_textList[ps];
}

void TextureCache::invalidateTextures()
{
	for(map<string, Tex*>::iterator i = m_textList.begin(); i != m_textList.end(); ++i)
		(*i).second->invalidate();
	
	for(map<string, Tex*>::iterator i = m_imgList.begin(); i != m_imgList.end(); ++i)
		(*i).second->invalidate();
}

wxFont* TextureCache::getFont(unsigned int pixelSize)
{
#ifdef WIN32	
	return wxFont::New(wxSize(pixelSize, 0), wxFONTFAMILY_DEFAULT);
#else
	return 0;
#endif
}


TextureCache* TextureCache::_instance = 0;

TextureCache* TextureCache::instance()
{
	if(_instance == 0)
		_instance = new TextureCache();

	return _instance;
}

TextureCache::TextureCache()
{

}

TextureCache::~TextureCache()
{
	
	// TODO: Use STL algorithms
	while(!m_textList.empty())
	{
		delete m_textList.begin()->second;
		m_textList.erase(m_textList.begin());
    }

	while(!m_imgList.empty())
	{
		delete m_imgList.begin()->second;
		m_imgList.erase(m_imgList.begin());
    }
	_instance = 0;
}
