/**************************************************************************
 *	Copyright (C) 2004-2006  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "fe_include.hpp"
#include "ui_nmap_ctl.hpp"

void FeNmapCtl::OnScan(wxCommandEvent& event)
{
	wxString scanOption = m_ScanOptions->GetString(m_ScanOptions->GetSelection());
	wxString scanCL = scanOption.Remove(0, scanOption.Find('[') + 1).RemoveLast();

	std::vector<wxString> *command = new std::vector<wxString>();

	
//	command->push_back(scanCL.ToAscii().data());
	command->push_back(scanCL);

	// Various Options
	if(this->m_ServiceInfo->IsChecked())
		command->push_back(wxT("-sV"));

	if(this->m_PingZero->IsChecked())
		command->push_back(wxT("-P0"));

	if(this->m_OSDetect->IsChecked())
		command->push_back(wxT("-O"));

	command->push_back(this->m_HostName->GetValue());

	m_WxConfig->Write(_("/nmap/dialog/HostName"), m_HostName->GetValue());
	m_WxConfig->Write(_("/nmap/dialog/ScanOptions"), m_ScanOptions->GetSelection());
	
	m_WxConfig->Write(_("/nmap/dialog/ServiceInfo"), m_ServiceInfo->IsChecked());
	m_WxConfig->Write(_("/nmap/dialog/PingZero"), m_PingZero->IsChecked());
	m_WxConfig->Write(_("/nmap/dialog/OSDetect"), m_OSDetect->IsChecked());
	
	FeTool *tool = m_Core->spawnNmap(command, m_HostName->GetValue());
	
	if(tool == 0)
	{
		wxMessageDialog dialog(m_Parent, _("Nmap Launch failed\nCheck the path"), _("Launch Failed"), wxOK | wxICON_ERROR);
		dialog.ShowModal();		
	}

//	m_NmapDialog->m_ScanOptions
/*
	command->push_back("-O");
	command->push_back("192.168.1.*");
	
*/
	event.Skip();
}


void FeNmapCtl::OnScanTypeSelect(wxCommandEvent& event)
{
	event.Skip();
}

//	void OnCancel(wxCommandEvent& event);
void FeNmapCtl::OnHostText(wxCommandEvent& event)
{
	// TODO check if it's not just blank
	m_Scan->Enable(m_HostName->GetValue().length() > 0);

	event.Skip();
}

void FeNmapCtl::OnHostTextEnter(wxCommandEvent& event)
{
	OnScan(event);
}


FeNmapCtl::FeNmapCtl(wxWindow *parent, FeCore *core)
 : NmapDialog(parent), m_Parent(parent), m_Core(core)
{

	m_WxConfig = m_Core->getConfig()->getWxConfig();
	
	wxString readVal;
	int scanOpt = 0;
	bool option = false;
	
	m_WxConfig->Read(_("/nmap/dialog/HostName"), &readVal, wxT(""));
	m_HostName->SetValue(readVal);	

	m_WxConfig->Read(_("/nmap/dialog/ScanOptions"), &scanOpt, 1);
	m_ScanOptions->SetSelection(scanOpt);
	
	m_WxConfig->Read(_("/nmap/dialog/ServiceInfo"), &option, false);
	m_ServiceInfo->SetValue(option);
	
	m_WxConfig->Read(_("/nmap/dialog/PingZero"), &option, false);
	m_PingZero->SetValue(option);
	
	m_WxConfig->Read(_("/nmap/dialog/OSDetect"), &option, true);
	m_OSDetect->SetValue(option);
	
	m_Scan->Enable(m_HostName->GetValue().length() > 0);

#ifndef WIN32
 	// Disable features of nmap that require root
 	if(geteuid() != 0)
 	{
		// Show warnings
		#include "warning.xpm"
		wxBitmap bmp(warning_xpm);
		m_WarningBitmap->SetBitmap(bmp);
		m_WarningBitmap->Show();
		m_NonRootWarning->Show();
	
 		m_ScanOptions->Enable(0, false);
 		m_ScanOptions->Enable(2, false);
 		m_ScanOptions->Enable(3, false);		
 		m_ScanOptions->Enable(4, false);
 		m_ScanOptions->Enable(5, false);		
 		m_ScanOptions->Enable(6, false);
 		m_ScanOptions->Enable(7, false);
 		m_ScanOptions->Enable(8, false);
 		m_ScanOptions->Enable(9, false);
		m_OSDetect->SetValue(false);
		m_OSDetect->Enable(false);
	
		wxSizer* sizer = this->GetSizer();
		sizer->Fit(this);
 	}
 
#endif	

}
