#include "fe_include.hpp"

#include "wxfe3d_app.hpp"
#include "wxfe3d_frame.hpp"

/*------------------------------------------------------------------
  Application object ( equivalent to main() )
------------------------------------------------------------------ */

IMPLEMENT_APP(Fe3DApp)

bool Fe3DApp::OnInit()
{
	
	if (!wxApp::OnInit())
		return false;
	
	// We need all image handlers for texture maps.
	::wxInitAllImageHandlers();

	m_Core = new FeCore();

	if(!m_Core->init())
	{
//		cerr<<"Failed to initilize the core"<<endl;
		::wxMessageBox(_("The Core failed the initialize,this probably means fe3d can't find the data files."),
					_("Failed to Initialize Core"), wxOK | wxICON_ERROR, 0);
		return false;
	}

	for(unsigned int i = 0; i < m_CmdFiles.size(); ++i)
	{
		wxString scanFile = m_CmdFiles[i];
		
#ifdef UNICODE
	FeHostsNode* sNode = new FeHostsNode(scanFile.ToAscii().data(), 0);
	m_Core->loadScan(scanFile.ToAscii().data(), sNode);
#else
	FeHostsNode* sNode = new FeHostsNode(scanFile.c_str(), 0);
	m_Core->loadScan(scanFile.c_str(), sNode);
#endif		
		
	}
    m_Frame = Fe3DFrame::Create(NULL, m_Core);
    return true;
}

int Fe3DApp::OnExit(){
	
	if(m_Core){
		delete m_Core;
		m_Core = 0;
	}
	return 0;
}

void Fe3DApp::OnInitCmdLine(wxCmdLineParser& parser)
{
    parser.SetDesc (g_cmdLineDesc);
    // must refuse '/' as parameter starter or cannot use "/path" style paths
    parser.SetSwitchChars (wxT("-"));
}

bool Fe3DApp::OnCmdLineParsed(wxCmdLineParser& parser)
{
	int count = parser.GetParamCount();
	
	for(unsigned int i = 0; i < parser.GetParamCount(); ++i)
	{
		m_CmdFiles.Add(parser.GetParam(i));
	}
	
	return true;
}
