/**************************************************************************
 *	Copyright (C) 2004-2006  iCapsid.net
 *
 * 	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, 
 *	Boston, MA  02111-1307, USA.
 *
 ****************************************************************************/

#include "fe_include.hpp"
#include "wxfe3d_glcanvas.hpp"
#include "ui_nmap_ctl.hpp"

enum
{
	Main_New		= wxID_NEW,
	Main_Open		= wxID_OPEN,
	Main_Close		= wxID_CLOSE,
	Main_Save		= wxID_SAVE,
	Main_SaveAs		= wxID_SAVEAS,
	Main_Quit		= wxID_EXIT,
	Main_Settings	= 1,

	Main_Launch,
	Main_Clear,

	Help_About		= wxID_ABOUT
};

BEGIN_EVENT_TABLE(Fe3DGLCanvas, wxGLCanvas)
	// Main Events
    EVT_SIZE(Fe3DGLCanvas::OnSize)
    EVT_PAINT(Fe3DGLCanvas::OnPaint)
    EVT_ERASE_BACKGROUND(Fe3DGLCanvas::OnEraseBackground)
    EVT_ENTER_WINDOW(Fe3DGLCanvas::OnEnterWindow)
	EVT_TIMER(wxID_ANY, Fe3DGLCanvas::OnTimer)

	// Menu Events
	EVT_MENU(Main_Open,  Fe3DGLCanvas::OnMainOpen)
	EVT_MENU(Main_Launch,  Fe3DGLCanvas::OnMainLaunch)
	EVT_MENU(Main_Save,  Fe3DGLCanvas::OnMainSave)
	
//	EVT_MENU(File_SaveAs,  Fe3DGLCanvas::OnMainSaveAs)
//	EVT_MENU_RANGE(wxID_FILE1, wxID_FILE9, DohGLFrame::OnFileRecent)
//	EVT_MENU(File_Settings,  DohGLFrame::OnMainSettings)
//  EVT_MENU(Main_Quit,  Fe3DGLCanvas::OnQuit)
	EVT_MENU(Main_Launch, Fe3DGLCanvas::OnMainLaunch)
	EVT_MENU(Main_Clear, Fe3DGLCanvas::OnMainClear)

	// Mouse Events
	EVT_MOTION(Fe3DGLCanvas::OnMouseMotion)
	EVT_LEFT_DOWN(Fe3DGLCanvas::OnMouseButtonLD)
	EVT_LEFT_UP(Fe3DGLCanvas::OnMouseButtonLU)
	EVT_RIGHT_DOWN(Fe3DGLCanvas::OnMouseButtonRD)
	EVT_RIGHT_UP(Fe3DGLCanvas::OnMouseButtonRU)
END_EVENT_TABLE()

Fe3DGLCanvas::Fe3DGLCanvas(wxWindow *parent, wxWindowID id,
    const wxPoint& pos, const wxSize& size, long style, const wxString& name)
    : wxGLCanvas(parent, (wxGLCanvas*) NULL, id, pos, size, style|wxFULL_REPAINT_ON_RESIZE , name )
{
	m_Init = false;
	m_Timer.SetOwner(this);
	m_Timer.Start(70);

	m_MainPopup.Append(Main_Open, wxT("Import Scan"), wxT("Open Project"));
	m_MainPopup.AppendSeparator();
	m_MainPopup.Append(Main_Launch, wxT("Launch Scan"), wxT("Launch nmap scan"));
	m_MainPopup.AppendSeparator();
//	m_MainPopup.Append(Main_Save, wxT("Save"), wxT("Save"));
	m_MainPopup.Append(Main_Clear, wxT("Clear Scene"), wxT("Clear Scene"));
	m_MainPopup.AppendSeparator();
	m_MainPopup.Append(Main_Quit, wxT("Exit\tAlt-F4"), wxT("Quit this program"));

//	SetDropTarget(m_DropTarget);
}

Fe3DGLCanvas::~Fe3DGLCanvas()
{

}

void Fe3DGLCanvas::InitGL()
{
    SetCurrent();

	m_Core->initGL();
}

void Fe3DGLCanvas::Render()
{
    wxPaintDC dc(this);

#ifndef __WXMOTIF__
    if (!GetContext()) return;
#endif

//	wxASSERT(m_Core != 0);

    SetCurrent();
    // Init OpenGL once, but after SetCurrent
    if (!m_Init)
    {
		InitGL();
        m_Init = true;
    }

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	m_Core->render(false);

	glFlush();
    SwapBuffers();
}

void Fe3DGLCanvas::RenderSelect(int x, int y)
{
	m_Core->enablePicking(x, y);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);
	
	m_Core->render(true);
	
	glEnable(GL_LIGHTING);
	glEnable(GL_TEXTURE_2D);
	m_Core->disablePicking();
}

void Fe3DGLCanvas::OnEnterWindow( wxMouseEvent& WXUNUSED(event) )
{
    SetFocus();
}

void Fe3DGLCanvas::OnPaint( wxPaintEvent& WXUNUSED(event) )
{
    Render();
}

void Fe3DGLCanvas::OnSize(wxSizeEvent& event)
{
    // this is also necessary to update the context on some platforms
    wxGLCanvas::OnSize(event);

    int w, h;
    GetClientSize(&w, &h);
    m_Core->getConfig()->setWindowSize(w, h);
    
    // set GL viewport (not called by wxGLCanvas::OnSize on all platforms...)
#ifndef __WXMOTIF__
    if (GetContext())
#endif
    {
        SetCurrent();
		m_Core->initGL();
        glViewport(0, 0, (GLint) w, (GLint) h);
    }
}

void Fe3DGLCanvas::OnEraseBackground(wxEraseEvent& WXUNUSED(event))
{
  // Do nothing, to avoid flashing.
}

void Fe3DGLCanvas::OnTimer(wxTimerEvent& event)
{
	// TODO : Stop the timer here
    m_Core->tick();

	this->Refresh();
}

void Fe3DGLCanvas::OnMainOpen(wxCommandEvent& WXUNUSED(event))
{
	wxFileDialog dialog(this, wxT("Choose Scan File"), wxT(""), wxT(""), wxT("*.xml"), wxFD_OPEN | wxFD_FILE_MUST_EXIST);

	if(dialog.ShowModal() != wxID_OK){
		return;
	}

	wxString fileName = dialog.GetPath();
//	m_FileHistory->AddFileToHistory(fileName);

	// TODO this should handle paths to extract the name better

	int start = fileName.Len() - 20;
	int end = fileName.Len();

	if(start < 0)
		start = 0;

#ifdef UNICODE
	FeHostsNode* sNode = new FeHostsNode(fileName.SubString(start,end).ToAscii().data(), 0);
#else	
	FeHostsNode* sNode = new FeHostsNode(fileName.SubString(start,end).c_str(), 0);
#endif
	m_Core->loadScan(fileName.ToAscii(), sNode);

	this->Refresh();
}

void Fe3DGLCanvas::OnMainLaunch(wxCommandEvent& WXUNUSED(event))
{
	FeNmapCtl *nmap_dialog = new FeNmapCtl(this, m_Core);

	nmap_dialog->Show();
}

void Fe3DGLCanvas::OnMainSave(wxCommandEvent& WXUNUSED(event))
{
	wxFileDialog dialog(this, wxT("Save fe3d file"), wxT(""), wxT(""), wxT("*.fe3d"), wxFD_SAVE | wxFD_OVERWRITE_PROMPT);

	if(dialog.ShowModal() == wxID_OK)
	{
//		m_Core->SetProjectFile(dialog.GetPath());
		m_Core->SaveFile(dialog.GetPath());

//		if(this->m_ShowFullPath)
//			this->SetTitle(wxString::Format(wxT("Debugger for OpenGL - %s"),dialog.GetPath()));
//		else
//			this->SetTitle(wxString::Format(wxT("Debugger for OpenGL - %s"),dialog.GetPath().AfterLast(wxT('\\'))));
	}
}

void Fe3DGLCanvas::OnMainClear(wxCommandEvent& event)
{
	m_Core->clearScene();
}

void Fe3DGLCanvas::OnQuit(wxCommandEvent& WXUNUSED(event))
{
    Close(true);
}

void Fe3DGLCanvas::OnMouseMotion(wxMouseEvent& event)
{
	if((m_Core->mouseDownLeft) && (!m_Core->underSelection))
	{
		// Pan Up/Down
		if(event.ShiftDown())
		{
			m_Core->y_Trans += 0.005f*(float)(m_Core->prevY - event.GetY());
		}
		else if(event.ControlDown())
		{ // Zoom In/Out
			m_Core->z_Trans -= 0.005f*(float)(m_Core->prevX - event.GetX());
		}
		else 
		{	// Rotate around
			m_Core->y_Rot -= 0.5f*(float)(m_Core->prevX - event.GetX());
		}
		
		m_Core->prevX = event.GetX();
		m_Core->prevY = event.GetY();
	}

//	this->RenderSelect(event.GetX(), event.GetY());
	this->Refresh();
}

void Fe3DGLCanvas::OnMouseButtonLD(wxMouseEvent& event)
{
	m_Core->mouseDownLeft = true;
	m_Core->prevX = event.GetX();
	m_Core->prevY = event.GetY();

	event.Skip();
}
void Fe3DGLCanvas::OnMouseButtonLU(wxMouseEvent& event)
{
	m_Core->mouseDownLeft = false;

	if(!event.Dragging())
		this->RenderSelect(event.GetX(), event.GetY());

	event.Skip();
}

void Fe3DGLCanvas::OnMouseButtonRD(wxMouseEvent& event)
{
	m_Core->mouseDownRight = true;
}

void Fe3DGLCanvas::OnMouseButtonRU(wxMouseEvent& event)
{
	m_Core->mouseDownRight = false;
	PopupMenu(&m_MainPopup, event.GetX(), event.GetY());
}	
